@@ -1,33 +1,48 @@
 <?php
 /**
- * ownCloud - trash bin
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Bastien Ho <bastienho@urbancube.fr>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Florin Peter <github@florin-peter.de>
+ * @author Georg Ehrke <georg@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Martin Mattel <martin.mattel@diemattels.at>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Qingping Hou <dave2008713@gmail.com>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
+ * @author Sjors van der Pluijm <sjors@desjors.nl>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Victor Dubiniuk <dubiniuk@owncloud.com>
+ * @author Vincent Petry <pvince81@owncloud.com>
  *
- * @author Bjoern Schiessle
- * @copyright 2013 Bjoern Schiessle schiessle@owncloud.com
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or any later version.
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 namespace OCA\Files_Trashbin;
 
 use OC\Files\Filesystem;
+use OC\Files\View;
+use OCA\Files_Trashbin\AppInfo\Application;
+use OCA\Files_Trashbin\Command\Expire;
 
 class Trashbin {
-	// how long do we keep files in the trash bin if no other value is defined in the config file (unit: days)
-
-	const DEFAULT_RETENTION_OBLIGATION = 30;
 
 	// unit: percentage; 50% of available disk space/quota
 	const DEFAULTMAXSIZE = 50;
@@ -39,6 +54,16 @@ class Trashbin {
 	 */
 	private static $scannedVersions = false;
 
+	/**
+	 * Ensure we dont need to scan the file during the move to trash
+	 * by triggering the scan in the pre-hook
+	 *
+	 * @param array $params
+	 */
+	public static function ensureFileScannedHook($params) {
+		self::getUidAndFilename($params['path']);
+	}
+
 	public static function getUidAndFilename($filename) {
 		$uid = \OC\Files\Filesystem::getOwner($filename);
 		\OC\Files\Filesystem::initMountPoints($uid);
@@ -109,31 +134,33 @@ class Trashbin {
 
 	/**
 	 * copy file to owners trash
+	 *
 	 * @param string $sourcePath
 	 * @param string $owner
-	 * @param string $ownerPath
+	 * @param $targetPath
+	 * @param $user
 	 * @param integer $timestamp
 	 */
-	private static function copyFilesToOwner($sourcePath, $owner, $ownerPath, $timestamp) {
+	private static function copyFilesToUser($sourcePath, $owner, $targetPath, $user, $timestamp) {
 		self::setUpTrash($owner);
 
-		$ownerFilename = basename($ownerPath);
-		$ownerLocation = dirname($ownerPath);
+		$targetFilename = basename($targetPath);
+		$targetLocation = dirname($targetPath);
 
 		$sourceFilename = basename($sourcePath);
 
 		$view = new \OC\Files\View('/');
 
-		$source = \OCP\User::getUser() . '/files_trashbin/files/' . $sourceFilename . '.d' . $timestamp;
-		$target = $owner . '/files_trashbin/files/' . $ownerFilename . '.d' . $timestamp;
+		$target = $user . '/files_trashbin/files/' . $targetFilename . '.d' . $timestamp;
+		$source = $owner . '/files_trashbin/files/' . $sourceFilename . '.d' . $timestamp;
 		self::copy_recursive($source, $target, $view);
 
 
 		if ($view->file_exists($target)) {
 			$query = \OC_DB::prepare("INSERT INTO `*PREFIX*files_trash` (`id`,`timestamp`,`location`,`user`) VALUES (?,?,?,?)");
-			$result = $query->execute(array($ownerFilename, $timestamp, $ownerLocation, $owner));
+			$result = $query->execute(array($targetFilename, $timestamp, $targetLocation, $user));
 			if (!$result) {
-				\OC_Log::write('files_trashbin', 'trash bin database couldn\'t be updated for the files owner', \OC_log::ERROR);
+				\OCP\Util::writeLog('files_trashbin', 'trash bin database couldn\'t be updated for the files owner', \OCP\Util::ERROR);
 			}
 		}
 	}
@@ -143,17 +170,17 @@ class Trashbin {
 	 * move file to the trash bin
 	 *
 	 * @param string $file_path path to the deleted file/directory relative to the files root directory
+	 * @return bool
 	 */
 	public static function move2trash($file_path) {
 		// get the user for which the filesystem is setup
 		$root = Filesystem::getRoot();
 		list(, $user) = explode('/', $root);
-		$size = 0;
 		list($owner, $ownerPath) = self::getUidAndFilename($file_path);
 
-		$view = new \OC\Files\View('/' . $user);
+		$ownerView = new \OC\Files\View('/' . $owner);
 		// file has been deleted in between
-		if (is_null($ownerPath) || $ownerPath === '' || !$view->file_exists('/files/' . $file_path)) {
+		if (is_null($ownerPath) || $ownerPath === '' || !$ownerView->file_exists('/files/' . $ownerPath)) {
 			return true;
 		}
 
@@ -163,65 +190,62 @@ class Trashbin {
 			self::setUpTrash($owner);
 		}
 
-		$path_parts = pathinfo($file_path);
+		$path_parts = pathinfo($ownerPath);
 
 		$filename = $path_parts['basename'];
 		$location = $path_parts['dirname'];
 		$timestamp = time();
 
-		$userTrashSize = self::getTrashbinSize($user);
-
 		// disable proxy to prevent recursive calls
-		$proxyStatus = \OC_FileProxy::$enabled;
-		\OC_FileProxy::$enabled = false;
 		$trashPath = '/files_trashbin/files/' . $filename . '.d' . $timestamp;
+
+		/** @var \OC\Files\Storage\Storage $trashStorage */
+		list($trashStorage, $trashInternalPath) = $ownerView->resolvePath($trashPath);
+		/** @var \OC\Files\Storage\Storage $sourceStorage */
+		list($sourceStorage, $sourceInternalPath) = $ownerView->resolvePath('/files/' . $ownerPath);
 		try {
-			$sizeOfAddedFiles = $view->filesize('/files/' . $file_path);
-			if ($view->file_exists($trashPath)) {
-				$view->unlink($trashPath);
+			$sizeOfAddedFiles = $sourceStorage->filesize($sourceInternalPath);
+			if ($trashStorage->file_exists($trashInternalPath)) {
+				$trashStorage->unlink($trashInternalPath);
 			}
-			$view->rename('/files/' . $file_path, $trashPath);
+			$trashStorage->moveFromStorage($sourceStorage, $sourceInternalPath, $trashInternalPath);
 		} catch (\OCA\Files_Trashbin\Exceptions\CopyRecursiveException $e) {
 			$sizeOfAddedFiles = false;
-			if ($view->file_exists($trashPath)) {
-				$view->deleteAll($trashPath);
+			if ($trashStorage->file_exists($trashInternalPath)) {
+				$trashStorage->unlink($trashInternalPath);
 			}
-			\OC_Log::write('files_trashbin', 'Couldn\'t move ' . $file_path . ' to the trash bin', \OC_log::ERROR);
+			\OCP\Util::writeLog('files_trashbin', 'Couldn\'t move ' . $file_path . ' to the trash bin', \OCP\Util::ERROR);
 		}
-		\OC_FileProxy::$enabled = $proxyStatus;
 
-		if ($view->file_exists('/files/' . $file_path)) { // failed to delete the original file, abort
-			$view->unlink($trashPath);
+		if ($sourceStorage->file_exists($sourceInternalPath)) { // failed to delete the original file, abort
+			$sourceStorage->unlink($sourceInternalPath);
 			return false;
 		}
 
+		$ownerView->getUpdater()->rename('/files/' . $ownerPath, $trashPath);
+
 		if ($sizeOfAddedFiles !== false) {
-			$size = $sizeOfAddedFiles;
 			$query = \OC_DB::prepare("INSERT INTO `*PREFIX*files_trash` (`id`,`timestamp`,`location`,`user`) VALUES (?,?,?,?)");
-			$result = $query->execute(array($filename, $timestamp, $location, $user));
+			$result = $query->execute(array($filename, $timestamp, $location, $owner));
 			if (!$result) {
-				\OC_Log::write('files_trashbin', 'trash bin database couldn\'t be updated', \OC_log::ERROR);
+				\OCP\Util::writeLog('files_trashbin', 'trash bin database couldn\'t be updated', \OCP\Util::ERROR);
 			}
 			\OCP\Util::emitHook('\OCA\Files_Trashbin\Trashbin', 'post_moveToTrash', array('filePath' => \OC\Files\Filesystem::normalizePath($file_path),
 				'trashPath' => \OC\Files\Filesystem::normalizePath($filename . '.d' . $timestamp)));
 
-			$size += self::retainVersions($file_path, $filename, $owner, $ownerPath, $timestamp);
-			$size += self::retainEncryptionKeys($file_path, $filename, $timestamp);
+			self::retainVersions($filename, $owner, $ownerPath, $timestamp);
 
 			// if owner !== user we need to also add a copy to the owners trash
 			if ($user !== $owner) {
-				self::copyFilesToOwner($file_path, $owner, $ownerPath, $timestamp);
+				self::copyFilesToUser($ownerPath, $owner, $file_path, $user, $timestamp);
 			}
 		}
 
-		$userTrashSize += $size;
-		$userTrashSize -= self::expire($userTrashSize, $user);
+		self::scheduleExpire($user);
 
 		// if owner !== user we also need to update the owners trash size
 		if ($owner !== $user) {
-			$ownerTrashSize = self::getTrashbinSize($owner);
-			$ownerTrashSize += $size;
-			$ownerTrashSize -= self::expire($ownerTrashSize, $owner);
+			self::scheduleExpire($owner);
 		}
 
 		return ($sizeOfAddedFiles === false) ? false : true;
@@ -230,7 +254,6 @@ class Trashbin {
 	/**
 	 * Move file versions to trash so that they can be restored later
 	 *
-	 * @param string $file_path path to original file
 	 * @param string $filename of deleted file
 	 * @param string $owner owner user id
 	 * @param string $ownerPath path relative to the owner's home storage
@@ -238,14 +261,10 @@ class Trashbin {
 	 *
 	 * @return int size of stored versions
 	 */
-	private static function retainVersions($file_path, $filename, $owner, $ownerPath, $timestamp) {
+	private static function retainVersions($filename, $owner, $ownerPath, $timestamp) {
 		$size = 0;
 		if (\OCP\App::isEnabled('files_versions') && !empty($ownerPath)) {
 
-			// disable proxy to prevent recursive calls
-			$proxyStatus = \OC_FileProxy::$enabled;
-			\OC_FileProxy::$enabled = false;
-
 			$user = \OCP\User::getUser();
 			$rootView = new \OC\Files\View('/');
 
@@ -254,80 +273,77 @@ class Trashbin {
 				if ($owner !== $user) {
 					self::copy_recursive($owner . '/files_versions/' . $ownerPath, $owner . '/files_trashbin/versions/' . basename($ownerPath) . '.d' . $timestamp, $rootView);
 				}
-				$rootView->rename($owner . '/files_versions/' . $ownerPath, $user . '/files_trashbin/versions/' . $filename . '.d' . $timestamp);
+				self::move($rootView, $owner . '/files_versions/' . $ownerPath, $user . '/files_trashbin/versions/' . $filename . '.d' . $timestamp);
 			} else if ($versions = \OCA\Files_Versions\Storage::getVersions($owner, $ownerPath)) {
+
 				foreach ($versions as $v) {
-					$size += $rootView->filesize($owner . '/files_versions' . $v['path'] . '.v' . $v['version']);
+					$size += $rootView->filesize($owner . '/files_versions/' . $v['path'] . '.v' . $v['version']);
 					if ($owner !== $user) {
-						$rootView->copy($owner . '/files_versions' . $v['path'] . '.v' . $v['version'], $owner . '/files_trashbin/versions/' . $v['name'] . '.v' . $v['version'] . '.d' . $timestamp);
+						self::copy($rootView, $owner . '/files_versions' . $v['path'] . '.v' . $v['version'], $owner . '/files_trashbin/versions/' . $v['name'] . '.v' . $v['version'] . '.d' . $timestamp);
 					}
-					$rootView->rename($owner . '/files_versions' . $v['path'] . '.v' . $v['version'], $user . '/files_trashbin/versions/' . $filename . '.v' . $v['version'] . '.d' . $timestamp);
+					self::move($rootView, $owner . '/files_versions' . $v['path'] . '.v' . $v['version'], $user . '/files_trashbin/versions/' . $filename . '.v' . $v['version'] . '.d' . $timestamp);
 				}
 			}
-
-			// enable proxy
-			\OC_FileProxy::$enabled = $proxyStatus;
 		}
 
 		return $size;
 	}
 
 	/**
-	 * Move encryption keys to trash so that they can be restored later
-	 *
-	 * @param string $file_path path to original file
-	 * @param string $filename of deleted file
-	 * @param integer $timestamp when the file was deleted
-	 *
-	 * @return int size of encryption keys
-	 */
-	private static function retainEncryptionKeys($file_path, $filename, $timestamp) {
-		$size = 0;
-
-		if (\OCP\App::isEnabled('files_encryption')) {
-
-			$user = \OCP\User::getUser();
-			$rootView = new \OC\Files\View('/');
-
-			list($owner, $ownerPath) = self::getUidAndFilename($file_path);
-
-			// file has been deleted in between
-			if (empty($ownerPath)) {
-				return 0;
-			}
-
-			$util = new \OCA\Files_Encryption\Util($rootView, $user);
-
-			$baseDir = '/files_encryption/';
-			if (!$util->isSystemWideMountPoint($ownerPath, $owner)) {
-				$baseDir = $owner . $baseDir;
-			}
-
-			$keyfiles = \OC\Files\Filesystem::normalizePath($baseDir . '/keys/' . $ownerPath);
-
-			if ($rootView->is_dir($keyfiles)) {
-				$size += self::calculateSize(new \OC\Files\View($keyfiles));
-				if ($owner !== $user) {
-					self::copy_recursive($keyfiles, $owner . '/files_trashbin/keys/' . basename($ownerPath) . '.d' . $timestamp, $rootView);
-				}
-				$rootView->rename($keyfiles, $user . '/files_trashbin/keys/' . $filename . '.d' . $timestamp);
-			}
-
-		}
-		return $size;
-	}
-
-	/**
-	 * restore files from trash bin
-	 *
-	 * @param string $file path to the deleted file
-	 * @param string $filename name of the file
-	 * @param int $timestamp time when the file was deleted
+	 * Move a file or folder on storage level
 	 *
+	 * @param View $view
+	 * @param string $source
+	 * @param string $target
 	 * @return bool
 	 */
-	public static function restore($file, $filename, $timestamp) {
+	private static function move(View $view, $source, $target) {
+		/** @var \OC\Files\Storage\Storage $sourceStorage */
+		list($sourceStorage, $sourceInternalPath) = $view->resolvePath($source);
+		/** @var \OC\Files\Storage\Storage $targetStorage */
+		list($targetStorage, $targetInternalPath) = $view->resolvePath($target);
+		/** @var \OC\Files\Storage\Storage $ownerTrashStorage */
 
+		$result = $targetStorage->moveFromStorage($sourceStorage, $sourceInternalPath, $targetInternalPath);
+		if ($result) {
+			$view->getUpdater()->rename($source, $target);
+		}
+		return $result;
+	}
+
+	/**
+	 * Copy a file or folder on storage level
+	 *
+	 * @param View $view
+	 * @param string $source
+	 * @param string $target
+	 * @return bool
+	 */
+	private static function copy(View $view, $source, $target) {
+		/** @var \OC\Files\Storage\Storage $sourceStorage */
+		list($sourceStorage, $sourceInternalPath) = $view->resolvePath($source);
+		/** @var \OC\Files\Storage\Storage $targetStorage */
+		list($targetStorage, $targetInternalPath) = $view->resolvePath($target);
+		/** @var \OC\Files\Storage\Storage $ownerTrashStorage */
+
+		$result = $targetStorage->copyFromStorage($sourceStorage, $sourceInternalPath, $targetInternalPath);
+		if ($result) {
+			$view->getUpdater()->update($target);
+		}
+		return $result;
+	}
+
+	/**
+	 * Restore a file or folder from trash bin
+	 *
+	 * @param string $file path to the deleted file/folder relative to "files_trashbin/files/",
+	 * including the timestamp suffix ".d12345678"
+	 * @param string $filename name of the file/folder
+	 * @param int $timestamp time when the file/folder was deleted
+	 *
+	 * @return bool true on success, false otherwise
+	 */
+	public static function restore($file, $filename, $timestamp) {
 		$user = \OCP\User::getUser();
 		$view = new \OC\Files\View('/' . $user);
 
@@ -335,12 +351,12 @@ class Trashbin {
 		if ($timestamp) {
 			$location = self::getLocation($user, $filename, $timestamp);
 			if ($location === false) {
-				\OC_Log::write('files_trashbin', 'trash bin database inconsistent!', \OC_Log::ERROR);
+				\OCP\Util::writeLog('files_trashbin', 'trash bin database inconsistent!', \OCP\Util::ERROR);
 			} else {
 				// if location no longer exists, restore file in the root directory
 				if ($location !== '/' &&
 					(!$view->is_dir('files/' . $location) ||
-					!$view->isCreatable('files/' . $location))
+						!$view->isCreatable('files/' . $location))
 				) {
 					$location = '';
 				}
@@ -352,12 +368,11 @@ class Trashbin {
 
 		$source = \OC\Files\Filesystem::normalizePath('files_trashbin/files/' . $file);
 		$target = \OC\Files\Filesystem::normalizePath('files/' . $location . '/' . $uniqueFilename);
+		if (!$view->file_exists($source)) {
+			return false;
+		}
 		$mtime = $view->filemtime($source);
 
-		// disable proxy to prevent recursive calls
-		$proxyStatus = \OC_FileProxy::$enabled;
-		\OC_FileProxy::$enabled = false;
-
 		// restore file
 		$restoreResult = $view->rename($source, $target);
 
@@ -371,22 +386,15 @@ class Trashbin {
 				'trashPath' => \OC\Files\Filesystem::normalizePath($file)));
 
 			self::restoreVersions($view, $file, $filename, $uniqueFilename, $location, $timestamp);
-			self::restoreEncryptionKeys($view, $file, $filename, $uniqueFilename, $location, $timestamp);
 
 			if ($timestamp) {
 				$query = \OC_DB::prepare('DELETE FROM `*PREFIX*files_trash` WHERE `user`=? AND `id`=? AND `timestamp`=?');
 				$query->execute(array($user, $filename, $timestamp));
 			}
 
-			// enable proxy
-			\OC_FileProxy::$enabled = $proxyStatus;
-
 			return true;
 		}
 
-		// enable proxy
-		\OC_FileProxy::$enabled = $proxyStatus;
-
 		return false;
 	}
 
@@ -404,9 +412,6 @@ class Trashbin {
 	private static function restoreVersions(\OC\Files\View $view, $file, $filename, $uniqueFilename, $location, $timestamp) {
 
 		if (\OCP\App::isEnabled('files_versions')) {
-			// disable proxy to prevent recursive calls
-			$proxyStatus = \OC_FileProxy::$enabled;
-			\OC_FileProxy::$enabled = false;
 
 			$user = \OCP\User::getUser();
 			$rootView = new \OC\Files\View('/');
@@ -417,7 +422,6 @@ class Trashbin {
 
 			// file has been deleted in between
 			if (empty($ownerPath)) {
-				\OC_FileProxy::$enabled = $proxyStatus;
 				return false;
 			}
 
@@ -429,7 +433,7 @@ class Trashbin {
 
 			if ($view->is_dir('/files_trashbin/versions/' . $file)) {
 				$rootView->rename(\OC\Files\Filesystem::normalizePath($user . '/files_trashbin/versions/' . $file), \OC\Files\Filesystem::normalizePath($owner . '/files_versions/' . $ownerPath));
-			} else if ($versions = self::getVersionsFromTrash($versionedFile, $timestamp)) {
+			} else if ($versions = self::getVersionsFromTrash($versionedFile, $timestamp, $user)) {
 				foreach ($versions as $v) {
 					if ($timestamp) {
 						$rootView->rename($user . '/files_trashbin/versions/' . $versionedFile . '.v' . $v . '.d' . $timestamp, $owner . '/files_versions/' . $ownerPath . '.v' . $v);
@@ -438,63 +442,6 @@ class Trashbin {
 					}
 				}
 			}
-
-			// enable proxy
-			\OC_FileProxy::$enabled = $proxyStatus;
-		}
-	}
-
-	/**
-	 * restore encryption keys from trash bin
-	 *
-	 * @param \OC\Files\View $view
-	 * @param string $file complete path to file
-	 * @param string $filename name of file
-	 * @param string $uniqueFilename new file name to restore the file without overwriting existing files
-	 * @param string $location location of file
-	 * @param int $timestamp deletion time
-	 * @return bool
-	 */
-	private static function restoreEncryptionKeys(\OC\Files\View $view, $file, $filename, $uniqueFilename, $location, $timestamp) {
-
-		if (\OCP\App::isEnabled('files_encryption')) {
-			$user = \OCP\User::getUser();
-			$rootView = new \OC\Files\View('/');
-
-			$target = \OC\Files\Filesystem::normalizePath('/' . $location . '/' . $uniqueFilename);
-
-			list($owner, $ownerPath) = self::getUidAndFilename($target);
-
-			// file has been deleted in between
-			if (empty($ownerPath)) {
-				return false;
-			}
-
-			$util = new \OCA\Files_Encryption\Util($rootView, $user);
-
-			$baseDir = '/files_encryption/';
-			if (!$util->isSystemWideMountPoint($ownerPath, $owner)) {
-				$baseDir = $owner . $baseDir;
-			}
-
-			$source_location = dirname($file);
-
-			if ($view->is_dir('/files_trashbin/keys/' . $file)) {
-				if ($source_location != '.') {
-					$keyfile = \OC\Files\Filesystem::normalizePath($user . '/files_trashbin/keys/' . $source_location . '/' . $filename);
-				} else {
-					$keyfile = \OC\Files\Filesystem::normalizePath($user . '/files_trashbin/keys/' . $filename);
-				}
-			}
-
-			if ($timestamp) {
-				$keyfile .= '.d' . $timestamp;
-			}
-
-			if ($rootView->is_dir($keyfile)) {
-				$rootView->rename($keyfile, $baseDir . '/keys/' . $ownerPath);
-			}
-
 		}
 	}
 
@@ -507,11 +454,12 @@ class Trashbin {
 		$view->deleteAll('files_trashbin');
 		$query = \OC_DB::prepare('DELETE FROM `*PREFIX*files_trash` WHERE `user`=?');
 		$query->execute(array($user));
+		$view->mkdir('files_trashbin');
+		$view->mkdir('files_trashbin/files');
 
 		return true;
 	}
 
-
 	/**
 	 * delete file from trash bin permanently
 	 *
@@ -533,8 +481,7 @@ class Trashbin {
 			$file = $filename;
 		}
 
-		$size += self::deleteVersions($view, $file, $filename, $timestamp);
-		$size += self::deleteEncryptionKeys($view, $file, $filename, $timestamp);
+		$size += self::deleteVersions($view, $file, $filename, $timestamp, $user);
 
 		if ($view->is_dir('/files_trashbin/files/' . $file)) {
 			$size += self::calculateSize(new \OC\Files\View('/' . $user . '/files_trashbin/files/' . $file));
@@ -555,14 +502,13 @@ class Trashbin {
 	 * @param $timestamp
 	 * @return int
 	 */
-	private static function deleteVersions(\OC\Files\View $view, $file, $filename, $timestamp) {
+	private static function deleteVersions(\OC\Files\View $view, $file, $filename, $timestamp, $user) {
 		$size = 0;
 		if (\OCP\App::isEnabled('files_versions')) {
-			$user = \OCP\User::getUser();
 			if ($view->is_dir('files_trashbin/versions/' . $file)) {
 				$size += self::calculateSize(new \OC\Files\view('/' . $user . '/files_trashbin/versions/' . $file));
 				$view->unlink('files_trashbin/versions/' . $file);
-			} else if ($versions = self::getVersionsFromTrash($filename, $timestamp)) {
+			} else if ($versions = self::getVersionsFromTrash($filename, $timestamp, $user)) {
 				foreach ($versions as $v) {
 					if ($timestamp) {
 						$size += $view->filesize('/files_trashbin/versions/' . $filename . '.v' . $v . '.d' . $timestamp);
@@ -577,32 +523,6 @@ class Trashbin {
 		return $size;
 	}
 
-	/**
-	 * @param \OC\Files\View $view
-	 * @param $file
-	 * @param $filename
-	 * @param $timestamp
-	 * @return int
-	 */
-	private static function deleteEncryptionKeys(\OC\Files\View $view, $file, $filename, $timestamp) {
-		$size = 0;
-		if (\OCP\App::isEnabled('files_encryption')) {
-			$user = \OCP\User::getUser();
-
-			$keyfiles = \OC\Files\Filesystem::normalizePath('files_trashbin/keys/' . $filename);
-
-			if ($timestamp) {
-				$keyfiles .= '.d' . $timestamp;
-			}
-			if ($view->is_dir($keyfiles)) {
-				$size += self::calculateSize(new \OC\Files\View('/' . $user . '/' . $keyfiles));
-				$view->deleteAll($keyfiles);
-
-			}
-		}
-		return $size;
-	}
-
 	/**
 	 * check to see whether a file exists in trashbin
 	 *
@@ -654,8 +574,9 @@ class Trashbin {
 		if ($quota === null || $quota === 'none') {
 			$quota = \OC\Files\Filesystem::free_space('/');
 			$softQuota = false;
-			if ($quota === \OCP\Files\FileInfo::SPACE_UNKNOWN) {
-				$quota = 0;
+			// inf or unknown free space
+			if ($quota < 0) {
+				$quota = PHP_INT_MAX;
 			}
 		} else {
 			$quota = \OCP\Util::computerFileSize($quota);
@@ -680,6 +601,7 @@ class Trashbin {
 
 	/**
 	 * resize trash bin if necessary after a new file was added to ownCloud
+	 *
 	 * @param string $user user id
 	 */
 	public static function resizeTrash($user) {
@@ -689,62 +611,63 @@ class Trashbin {
 		$freeSpace = self::calculateFreeSpace($size, $user);
 
 		if ($freeSpace < 0) {
-			self::expire($size, $user);
+			self::scheduleExpire($user);
 		}
 	}
 
 	/**
 	 * clean up the trash bin
 	 *
-	 * @param int $trashbinSize current size of the trash bin
 	 * @param string $user
-	 * @return int size of expired files
 	 */
-	private static function expire($trashbinSize, $user) {
-
-		// let the admin disable auto expire
-		$autoExpire = \OC_Config::getValue('trashbin_auto_expire', true);
-		if ($autoExpire === false) {
-			return 0;
-		}
-
-		$availableSpace = self::calculateFreeSpace($trashbinSize, $user);
+	public static function expire($user) {
+		$trashBinSize = self::getTrashbinSize($user);
+		$availableSpace = self::calculateFreeSpace($trashBinSize, $user);
 		$size = 0;
 
-		$retention_obligation = \OC_Config::getValue('trashbin_retention_obligation', self::DEFAULT_RETENTION_OBLIGATION);
-
-		$limit = time() - ($retention_obligation * 86400);
-
 		$dirContent = Helper::getTrashFiles('/', $user, 'mtime');
 
 		// delete all files older then $retention_obligation
-		list($delSize, $count) = self::deleteExpiredFiles($dirContent, $user, $limit, $retention_obligation);
+		list($delSize, $count) = self::deleteExpiredFiles($dirContent, $user);
 
 		$size += $delSize;
 		$availableSpace += $size;
 
 		// delete files from trash until we meet the trash bin size limit again
 		$size += self::deleteFiles(array_slice($dirContent, $count), $user, $availableSpace);
+	}
 
-		return $size;
+	/**
+	 * @param string $user
+	 */
+	private static function scheduleExpire($user) {
+		// let the admin disable auto expire
+		$application = new Application();
+		$expiration = $application->getContainer()->query('Expiration');
+		if ($expiration->isEnabled()) {
+			\OC::$server->getCommandBus()->push(new Expire($user));
+		}
 	}
 
 	/**
 	 * if the size limit for the trash bin is reached, we delete the oldest
 	 * files in the trash bin until we meet the limit again
+	 *
 	 * @param array $files
 	 * @param string $user
 	 * @param int $availableSpace available disc space
 	 * @return int size of deleted files
 	 */
 	protected static function deleteFiles($files, $user, $availableSpace) {
+		$application = new Application();
+		$expiration = $application->getContainer()->query('Expiration');
 		$size = 0;
 
 		if ($availableSpace < 0) {
 			foreach ($files as $file) {
-				if ($availableSpace < 0) {
+				if ($availableSpace < 0 && $expiration->isExpired($file['mtime'], true)) {
 					$tmp = self::delete($file['name'], $user, $file['mtime']);
-					\OC_Log::write('files_trashbin', 'remove "' . $file['name'] . '" (' . $tmp . 'B) to meet the limit of trash bin size (50% of available quota)', \OC_log::INFO);
+					\OCP\Util::writeLog('files_trashbin', 'remove "' . $file['name'] . '" (' . $tmp . 'B) to meet the limit of trash bin size (50% of available quota)', \OCP\Util::INFO);
 					$availableSpace += $tmp;
 					$size += $tmp;
 				} else {
@@ -760,20 +683,23 @@ class Trashbin {
 	 *
 	 * @param array $files list of files sorted by mtime
 	 * @param string $user
-	 * @param int $limit files older then limit should be deleted
-	 * @param int $retention_obligation max age of file in days
 	 * @return array size of deleted files and number of deleted files
 	 */
-	protected static function deleteExpiredFiles($files, $user, $limit, $retention_obligation) {
+	public static function deleteExpiredFiles($files, $user) {
+		$application = new Application();
+		$expiration = $application->getContainer()->query('Expiration');
 		$size = 0;
 		$count = 0;
 		foreach ($files as $file) {
 			$timestamp = $file['mtime'];
 			$filename = $file['name'];
-			if ($timestamp <= $limit) {
+			if ($expiration->isExpired($timestamp)) {
 				$count++;
 				$size += self::delete($filename, $user, $timestamp);
-				\OC_Log::write('files_trashbin', 'remove "' . $filename . '" from trash bin because it is older than ' . $retention_obligation, \OC_log::INFO);
+				\OC::$server->getLogger()->info(
+					'Remove "' . $filename . '" from trashbin because it exceeds max retention obligation term.',
+					['app' => 'files_trashbin']
+				);
 			} else {
 				break;
 			}
@@ -827,14 +753,14 @@ class Trashbin {
 	 * @param int $timestamp timestamp when the file was deleted
 	 * @return array
 	 */
-	private static function getVersionsFromTrash($filename, $timestamp) {
-		$view = new \OC\Files\View('/' . \OCP\User::getUser() . '/files_trashbin/versions');
+	private static function getVersionsFromTrash($filename, $timestamp, $user) {
+		$view = new \OC\Files\View('/' . $user . '/files_trashbin/versions');
 		$versions = array();
 
 		//force rescan of versions, local storage may not have updated the cache
 		if (!self::$scannedVersions) {
 			/** @var \OC\Files\Storage\Storage $storage */
-			list($storage, ) = $view->resolvePath('/');
+			list($storage,) = $view->resolvePath('/');
 			$storage->getScanner()->scan('files_trashbin/versions');
 			self::$scannedVersions = true;
 		}
@@ -874,6 +800,8 @@ class Trashbin {
 		$name = pathinfo($filename, PATHINFO_FILENAME);
 		$l = \OC::$server->getL10N('files_trashbin');
 
+		$location = '/' . trim($location, '/');
+
 		// if extension is not empty we set a dot in front of it
 		if ($ext !== '') {
 			$ext = '.' . $ext;
@@ -895,6 +823,7 @@ class Trashbin {
 
 	/**
 	 * get the size from a given root folder
+	 *
 	 * @param \OC\Files\View $view file view on the root folder
 	 * @return integer size of the folder
 	 */
@@ -906,7 +835,7 @@ class Trashbin {
 		$iterator = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($root), \RecursiveIteratorIterator::CHILD_FIRST);
 		$size = 0;
 
-                /**
+		/**
 		 * RecursiveDirectoryIterator on an NFS path isn't iterable with foreach
 		 * This bug is fixed in PHP 5.5.9 or before
 		 * See #8376
@@ -940,18 +869,20 @@ class Trashbin {
 	 */
 	public static function registerHooks() {
 		// create storage wrapper on setup
-		\OCP\Util::connectHook('OC_Filesystem', 'setup', 'OCA\Files_Trashbin\Storage', 'setupStorage');
+		\OCP\Util::connectHook('OC_Filesystem', 'preSetup', 'OCA\Files_Trashbin\Storage', 'setupStorage');
 		//Listen to delete user signal
 		\OCP\Util::connectHook('OC_User', 'pre_deleteUser', 'OCA\Files_Trashbin\Hooks', 'deleteUser_hook');
 		//Listen to post write hook
 		\OCP\Util::connectHook('OC_Filesystem', 'post_write', 'OCA\Files_Trashbin\Hooks', 'post_write_hook');
 		// pre and post-rename, disable trash logic for the copy+unlink case
+		\OCP\Util::connectHook('OC_Filesystem', 'delete', 'OCA\Files_Trashbin\Trashbin', 'ensureFileScannedHook');
 		\OCP\Util::connectHook('OC_Filesystem', 'rename', 'OCA\Files_Trashbin\Storage', 'preRenameHook');
 		\OCP\Util::connectHook('OC_Filesystem', 'post_rename', 'OCA\Files_Trashbin\Storage', 'postRenameHook');
 	}
 
 	/**
 	 * check if trash bin is empty for a given user
+	 *
 	 * @param string $user
 	 * @return bool
 	 */
@@ -960,7 +891,7 @@ class Trashbin {
 		$view = new \OC\Files\View('/' . $user . '/files_trashbin');
 		if ($view->is_dir('/files') && $dh = $view->opendir('/files')) {
 			while ($file = readdir($dh)) {
-				if ($file !== '.' and $file !== '..') {
+				if (!\OC\Files\Filesystem::isIgnoredDir($file)) {
 					return false;
 				}
 			}
@@ -973,6 +904,6 @@ class Trashbin {
 	 * @return string
 	 */
 	public static function preview_icon($path) {
-		return \OC_Helper::linkToRoute('core_ajax_trashbin_preview', array('x' => 36, 'y' => 36, 'file' => $path));
+		return \OCP\Util::linkToRoute('core_ajax_trashbin_preview', array('x' => 32, 'y' => 32, 'file' => $path));
 	}
 }
