@@ -12,6 +12,7 @@
  * @author Lukas Reschke <lukas@owncloud.com>
  * @author Lyonel Vincent <lyonel@ezix.org>
  * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Nicolas Grekas <nicolas.grekas@gmail.com>
  * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
  * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
  *
@@ -361,8 +362,9 @@ class Access extends LDAPUtility implements user\IUserTools {
 		$validGroupDNs = [];
 		foreach($groupDNs as $dn) {
 			$cacheKey = 'groupsMatchFilter-'.$dn;
-			if($this->connection->isCached($cacheKey)) {
-				if($this->connection->getFromCache($cacheKey)) {
+			$groupMatchFilter = $this->connection->getFromCache($cacheKey);
+			if(!is_null($groupMatchFilter)) {
+				if($groupMatchFilter) {
 					$validGroupDNs[] = $dn;
 				}
 				continue;
@@ -517,6 +519,7 @@ class Access extends LDAPUtility implements user\IUserTools {
 	private function ldap2ownCloudNames($ldapObjects, $isUsers) {
 		if($isUsers) {
 			$nameAttribute = $this->connection->ldapUserDisplayName;
+			$sndAttribute  = $this->connection->ldapUserDisplayName2;
 		} else {
 			$nameAttribute = $this->connection->ldapGroupDisplayName;
 		}
@@ -538,13 +541,14 @@ class Access extends LDAPUtility implements user\IUserTools {
 				if($isUsers) {
 					//cache the user names so it does not need to be retrieved
 					//again later (e.g. sharing dialogue).
-					$this->cacheUserExists($ocName);
-					if(!is_null($nameByLDAP)) {
-						$this->cacheUserDisplayName($ocName, $nameByLDAP);
+					if(is_null($nameByLDAP)) {
+						continue;
 					}
+					$sndName = isset($ldapObject[$sndAttribute][0])
+						? $ldapObject[$sndAttribute][0] : '';
+					$this->cacheUserDisplayName($ocName, $nameByLDAP, $sndName);
 				}
 			}
-			continue;
 		}
 		return $ownCloudNames;
 	}
@@ -571,8 +575,11 @@ class Access extends LDAPUtility implements user\IUserTools {
 	 * caches the user display name
 	 * @param string $ocName the internal ownCloud username
 	 * @param string $displayName the display name
+	 * @param string $displayName2 the second display name
 	 */
-	public function cacheUserDisplayName($ocName, $displayName) {
+	public function cacheUserDisplayName($ocName, $displayName, $displayName2 = '') {
+		$user = $this->userManager->get($ocName);
+		$displayName = $user->composeAndStoreDisplayName($displayName, $displayName2);
 		$cacheKeyTrunk = 'getDisplayName';
 		$this->connection->writeToCache($cacheKeyTrunk.$ocName, $displayName);
 	}
@@ -666,12 +673,25 @@ class Access extends LDAPUtility implements user\IUserTools {
 	 */
 	public function fetchUsersByLoginName($loginName, $attributes = array('dn')) {
 		$loginName = $this->escapeFilterPart($loginName);
-		$filter = \OCP\Util::mb_str_replace(
-			'%uid', $loginName, $this->connection->ldapLoginFilter, 'UTF-8');
+		$filter = str_replace('%uid', $loginName, $this->connection->ldapLoginFilter);
 		$users = $this->fetchListOfUsers($filter, $attributes);
 		return $users;
 	}
 
+	/**
+	 * counts the number of users according to a provided loginName and
+	 * utilizing the login filter.
+	 *
+	 * @param string $loginName
+	 * @return array
+	 */
+	public function countUsersByLoginName($loginName) {
+		$loginName = $this->escapeFilterPart($loginName);
+		$filter = str_replace('%uid', $loginName, $this->connection->ldapLoginFilter);
+		$users = $this->countUsers($filter);
+		return $users;
+	}
+
 	/**
 	 * @param string $filter
 	 * @param string|string[] $attr
@@ -692,27 +712,15 @@ class Access extends LDAPUtility implements user\IUserTools {
 	 * @param array $ldapRecords
 	 */
 	public function batchApplyUserAttributes(array $ldapRecords){
-		$displayNameAttribute = strtolower($this->connection->ldapUserDisplayName);
 		foreach($ldapRecords as $userRecord) {
-			if(!isset($userRecord[$displayNameAttribute])) {
-				// displayName is obligatory
-				continue;
-			}
-
-			$ocName  = $this->dn2ocname($userRecord['dn'][0], $userRecord[$displayNameAttribute]);
-			if(!$ocName) {
-				// no user name, skip.
-				continue;
-			}
+			$ocName  = $this->dn2ocname($userRecord['dn'][0]);
 			$this->cacheUserExists($ocName);
 			$user = $this->userManager->get($ocName);
 			if($user instanceof OfflineUser) {
 				$user->unmark();
 				$user = $this->userManager->get($ocName);
 			}
-			if(!is_null($user)) {
-				$user->processAttributes($userRecord);
-			}
+			$user->processAttributes($userRecord);
 		}
 	}
 
@@ -1069,7 +1077,7 @@ class Access extends LDAPUtility implements user\IUserTools {
 		$name = iconv('UTF-8', 'ASCII//TRANSLIT', $name);
 
 		// Replacements
-		$name = \OCP\Util::mb_str_replace(' ', '_', $name, 'UTF-8');
+		$name = str_replace(' ', '_', $name);
 
 		// Every remaining disallowed characters will be removed
 		$name = preg_replace('/[^a-zA-Z0-9_.@-]/u', '', $name);
@@ -1247,9 +1255,7 @@ class Access extends LDAPUtility implements user\IUserTools {
 		if(!$testConnection->setConfiguration($credentials)) {
 			return false;
 		}
-		$result=$testConnection->bind();
-		$this->connection->bind();
-		return $result;
+		return $testConnection->bind();
 	}
 
 	/**
@@ -1457,8 +1463,9 @@ class Access extends LDAPUtility implements user\IUserTools {
 	public function getSID($dn) {
 		$domainDN = $this->getDomainDNFromDN($dn);
 		$cacheKey = 'getSID-'.$domainDN;
-		if($this->connection->isCached($cacheKey)) {
-			return $this->connection->getFromCache($cacheKey);
+		$sid = $this->connection->getFromCache($cacheKey);
+		if(!is_null($sid)) {
+			return $sid;
 		}
 
 		$objectSid = $this->readAttribute($domainDN, 'objectsid');
