@@ -1,23 +1,22 @@
 <?php
-
 /**
- * ownCloud – LDAP User
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
  *
- * @author Arthur Schiwon
- * @copyright 2014 Arthur Schiwon blizzz@owncloud.com
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or any later version.
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -139,6 +138,85 @@ class User {
 		}
 	}
 
+	/**
+	 * processes results from LDAP for attributes as returned by getAttributesToRead()
+	 * @param array $ldapEntry the user entry as retrieved from LDAP
+	 */
+	public function processAttributes($ldapEntry) {
+		$this->markRefreshTime();
+		//Quota
+		$attr = strtolower($this->connection->ldapQuotaAttribute);
+		if(isset($ldapEntry[$attr])) {
+			$this->updateQuota($ldapEntry[$attr][0]);
+		}
+		unset($attr);
+
+		//Email
+		$attr = strtolower($this->connection->ldapEmailAttribute);
+		if(isset($ldapEntry[$attr])) {
+			$this->updateEmail($ldapEntry[$attr][0]);
+		}
+		unset($attr);
+
+		//displayName
+		$displayName = $displayName2 = '';
+		$attr = strtolower($this->connection->ldapUserDisplayName);
+		if(isset($ldapEntry[$attr])) {
+			$displayName = $ldapEntry[$attr][0];
+		}
+		$attr = strtolower($this->connection->ldapUserDisplayName2);
+		if(isset($ldapEntry[$attr])) {
+			$displayName2 = $ldapEntry[$attr][0];
+		}
+		if(!empty($displayName)) {
+			$this->composeAndStoreDisplayName($displayName);
+			$this->access->cacheUserDisplayName(
+				$this->getUsername(),
+				$displayName,
+				$displayName2
+			);
+		}
+		unset($attr);
+
+		// LDAP Username, needed for s2s sharing
+		if(isset($ldapEntry['uid'])) {
+			$this->storeLDAPUserName($ldapEntry['uid'][0]);
+		} else if(isset($ldapEntry['samaccountname'])) {
+			$this->storeLDAPUserName($ldapEntry['samaccountname'][0]);
+		}
+
+		//homePath
+		if(strpos($this->connection->homeFolderNamingRule, 'attr:') === 0) {
+			$attr = strtolower(substr($this->connection->homeFolderNamingRule, strlen('attr:')));
+			if(isset($ldapEntry[$attr])) {
+				$this->access->cacheUserHome(
+					$this->getUsername(), $this->getHomePath($ldapEntry[$attr][0]));
+			}
+		}
+
+		//memberOf groups
+		$cacheKey = 'getMemberOf'.$this->getUsername();
+		$groups = false;
+		if(isset($ldapEntry['memberof'])) {
+			$groups = $ldapEntry['memberof'];
+		}
+		$this->connection->writeToCache($cacheKey, $groups);
+
+		//Avatar
+		$attrs = array('jpegphoto', 'thumbnailphoto');
+		foreach ($attrs as $attr)  {
+			if(isset($ldapEntry[$attr])) {
+				$this->avatarImage = $ldapEntry[$attr][0];
+				// the call to the method that saves the avatar in the file
+				// system must be postponed after the login. It is to ensure
+				// external mounts are mounted properly (e.g. with login
+				// credentials from the session).
+				\OCP\Util::connectHook('OC_User', 'post_login', $this, 'updateAvatarPostLogin');
+				break;
+			}
+		}
+	}
+
 	/**
 	 * @brief returns the LDAP DN of the user
 	 * @return string
@@ -155,6 +233,69 @@ class User {
 		return $this->uid;
 	}
 
+	/**
+	 * returns the home directory of the user if specified by LDAP settings
+	 * @param string $valueFromLDAP
+	 * @return bool|string
+	 * @throws \Exception
+	 */
+	public function getHomePath($valueFromLDAP = null) {
+		$path = $valueFromLDAP;
+		$attr = null;
+
+		if(   is_null($path)
+		   && strpos($this->access->connection->homeFolderNamingRule, 'attr:') === 0
+		   && $this->access->connection->homeFolderNamingRule !== 'attr:')
+		{
+			$attr = substr($this->access->connection->homeFolderNamingRule, strlen('attr:'));
+			$homedir = $this->access->readAttribute(
+				$this->access->username2dn($this->getUsername()), $attr);
+			if ($homedir && isset($homedir[0])) {
+				$path = $homedir[0];
+			}
+		}
+
+		if(!empty($path)) {
+			//if attribute's value is an absolute path take this, otherwise append it to data dir
+			//check for / at the beginning or pattern c:\ resp. c:/
+			if(   '/' !== $path[0]
+			   && !(3 < strlen($path) && ctype_alpha($path[0])
+			       && $path[1] === ':' && ('\\' === $path[2] || '/' === $path[2]))
+			) {
+				$path = $this->config->getSystemValue('datadirectory',
+						\OC::$SERVERROOT.'/data' ) . '/' . $path;
+			}
+			//we need it to store it in the DB as well in case a user gets
+			//deleted so we can clean up afterwards
+			$this->config->setUserValue(
+				$this->getUsername(), 'user_ldap', 'homePath', $path
+			);
+			return $path;
+		}
+
+		if(    !is_null($attr)
+			&& $this->config->getAppValue('user_ldap', 'enforce_home_folder_naming_rule', true)
+		) {
+			// a naming rule attribute is defined, but it doesn't exist for that LDAP user
+			throw new \Exception('Home dir attribute can\'t be read from LDAP for uid: ' . $this->getUsername());
+		}
+
+		//false will apply default behaviour as defined and done by OC_User
+		$this->config->setUserValue($this->getUsername(), 'user_ldap', 'homePath', '');
+		return false;
+	}
+
+	public function getMemberOfGroups() {
+		$cacheKey = 'getMemberOf'.$this->getUsername();
+		$memberOfGroups = $this->connection->getFromCache($cacheKey);
+		if(!is_null($memberOfGroups)) {
+			return $memberOfGroups;
+		}
+		$groupDNs = $this->access->readAttribute($this->getDN(), 'memberOf');
+		$this->connection->writeToCache($cacheKey, $groupDNs);
+		return $groupDNs;
+	}
+
 	/**
 	 * @brief reads the image from LDAP that shall be used as Avatar
 	 * @return string data (provided by LDAP) | false
@@ -190,7 +331,7 @@ class User {
 	 * @brief marks the time when user features like email have been updated
 	 * @return null
 	 */
-	private function markRefreshTime() {
+	public function markRefreshTime() {
 		$this->config->setUserValue(
 			$this->uid, 'user_ldap', self::USER_PREFKEY_LASTREFRESH, time());
 	}
@@ -214,6 +355,7 @@ class User {
 
 	/**
 	 * Stores a key-value pair in relation to this user
+	 *
 	 * @param string $key
 	 * @param string $value
 	 */
@@ -222,11 +364,19 @@ class User {
 	}
 
 	/**
-	 * Stores the display name in the databae
+	 * Composes the display name and stores it in the database. The final
+	 * display name is returned.
+	 *
 	 * @param string $displayName
+	 * @param string $displayName2
+	 * @returns string the effective display name
 	 */
-	public function storeDisplayName($displayName) {
+	public function composeAndStoreDisplayName($displayName, $displayName2 = '') {
+		if(!empty($displayName2)) {
+			$displayName .= ' (' . $displayName2 . ')';
+		}
 		$this->store('displayName', $displayName);
+		return $displayName;
 	}
 
 	/**
@@ -253,48 +403,52 @@ class User {
 	}
 
 	/**
-	 * @brief fetches the email from LDAP and stores it as ownCloud user value
+	 * fetches the email from LDAP and stores it as ownCloud user value
+	 * @param string $valueFromLDAP if known, to save an LDAP read request
 	 * @return null
 	 */
-	public function updateEmail() {
+	public function updateEmail($valueFromLDAP = null) {
 		if($this->wasRefreshed('email')) {
 			return;
 		}
-
-		$email = null;
-		$emailAttribute = $this->connection->ldapEmailAttribute;
-		if(!empty($emailAttribute)) {
-			$aEmail = $this->access->readAttribute($this->dn, $emailAttribute);
-			if($aEmail && (count($aEmail) > 0)) {
-				$email = $aEmail[0];
-			}
-			if(!is_null($email)) {
-				$this->config->setUserValue(
-					$this->uid, 'settings', 'email', $email);
+		$email = $valueFromLDAP;
+		if(is_null($valueFromLDAP)) {
+			$emailAttribute = $this->connection->ldapEmailAttribute;
+			if(!empty($emailAttribute)) {
+				$aEmail = $this->access->readAttribute($this->dn, $emailAttribute);
+				if(is_array($aEmail) && (count($aEmail) > 0)) {
+					$email = $aEmail[0];
+				}
 			}
 		}
+		if(!is_null($email)) {
+			$this->config->setUserValue(
+				$this->uid, 'settings', 'email', $email);
+		}
 	}
 
 	/**
-	 * @brief fetches the quota from LDAP and stores it as ownCloud user value
+	 * fetches the quota from LDAP and stores it as ownCloud user value
+	 * @param string $valueFromLDAP the quota attribute's value can be passed,
+	 * to save the readAttribute request
 	 * @return null
 	 */
-	public function updateQuota() {
+	public function updateQuota($valueFromLDAP = null) {
 		if($this->wasRefreshed('quota')) {
 			return;
 		}
-
-		$quota = null;
+		//can be null
 		$quotaDefault = $this->connection->ldapQuotaDefault;
-		$quotaAttribute = $this->connection->ldapQuotaAttribute;
-		if(!empty($quotaDefault)) {
-			$quota = $quotaDefault;
-		}
-		if(!empty($quotaAttribute)) {
-			$aQuota = $this->access->readAttribute($this->dn, $quotaAttribute);
+		$quota = $quotaDefault !== '' ? $quotaDefault : null;
+		$quota = !is_null($valueFromLDAP) ? $valueFromLDAP : $quota;
 
-			if($aQuota && (count($aQuota) > 0)) {
-				$quota = $aQuota[0];
+		if(is_null($valueFromLDAP)) {
+			$quotaAttribute = $this->connection->ldapQuotaAttribute;
+			if(!empty($quotaAttribute)) {
+				$aQuota = $this->access->readAttribute($this->dn, $quotaAttribute);
+				if($aQuota && (count($aQuota) > 0)) {
+					$quota = $aQuota[0];
+				}
 			}
 		}
 		if(!is_null($quota)) {
@@ -302,6 +456,17 @@ class User {
 		}
 	}
 
+	/**
+	 * called by a post_login hook to save the avatar picture
+	 *
+	 * @param array $params
+	 */
+	public function updateAvatarPostLogin($params) {
+		if(isset($params['uid']) && $params['uid'] === $this->getUsername()) {
+			$this->updateAvatar();
+		}
+	}
+
 	/**
 	 * @brief attempts to get an image from LDAP and sets it as ownCloud avatar
 	 * @return null
@@ -342,8 +507,8 @@ class User {
 			$this->fs->setup($this->uid);
 		}
 
-		$avatar = $this->avatarManager->getAvatar($this->uid);
 		try {
+			$avatar = $this->avatarManager->getAvatar($this->uid);
 			$avatar->set($this->image);
 		} catch (\Exception $e) {
 			\OC::$server->getLogger()->notice(
