@@ -7,6 +7,7 @@
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Lukas Reschke <lukas@owncloud.com>
  * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Nicolas Grekas <nicolas.grekas@gmail.com>
  * @author Robin Appelman <icewind@owncloud.com>
  * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
  * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
@@ -630,13 +631,12 @@ class Wizard extends LDAPUtility {
 			throw new \Exception('missing placeholder');
 		}
 
-		$users = $this->access->fetchUsersByLoginName($loginName);
+		$users = $this->access->countUsersByLoginName($loginName);
 		if($this->ldap->errno($cr) !== 0) {
 			throw new \Exception($this->ldap->error($cr));
 		}
-		$filter = \OCP\Util::mb_str_replace(
-			'%uid', $loginName, $this->access->connection->ldapLoginFilter, 'UTF-8');
-		$this->result->addChange('ldap_test_loginname', count($users));
+		$filter = str_replace('%uid', $loginName, $this->access->connection->ldapLoginFilter);
+		$this->result->addChange('ldap_test_loginname', $users);
 		$this->result->addChange('ldap_test_effective_filter', $filter);
 		return $this->result;
 	}
@@ -666,12 +666,13 @@ class Wizard extends LDAPUtility {
 			//connectAndBind may throw Exception, it needs to be catched by the
 			//callee of this method
 
-			// unallowed anonymous bind throws 48. But if it throws 48, we
-			// detected port and TLS, i.e. it is successful.
 			try {
 				$settingsFound = $this->connectAndBind($p, $t);
 			} catch (\Exception $e) {
-				if($e->getCode() === 48) {
+				// any reply other than -1 (= cannot connect) is already okay,
+				// because then we found the server
+				// unavailable startTLS returns -11
+				if($e->getCode() > 0) {
 					$settingsFound = true;
 				} else {
 					throw $e;
@@ -1082,7 +1083,7 @@ class Wizard extends LDAPUtility {
 		} else if ($errNo === 2) {
 			return $this->connectAndBind($port, $tls, true);
 		}
-		throw new \Exception($error);
+		throw new \Exception($error, $errNo);
 	}
 
 	/**
