@@ -5,6 +5,7 @@
  * @author Björn Schießle <schiessle@owncloud.com>
  * @author Craig Morrissey <craig@owncloud.com>
  * @author dampfklon <me@dampfklon.de>
+ * @author Felix Böhm <felixboehm@gmx.de>
  * @author Joas Schilling <nickvergessen@owncloud.com>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Lukas Reschke <lukas@owncloud.com>
@@ -47,9 +48,28 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 					$shareType = (int)$_POST['shareType'];
 					$shareWith = $_POST['shareWith'];
 					$itemSourceName = isset($_POST['itemSourceName']) ? (string)$_POST['itemSourceName'] : null;
-					if ($shareType === OCP\Share::SHARE_TYPE_LINK && $shareWith == '') {
-						$shareWith = null;
+
+					/*
+					 * Nasty nasty fix for https://github.com/owncloud/core/issues/19950
+					 */
+					$passwordChanged = null;
+					if (is_array($shareWith)) {
+						$passwordChanged = ($shareWith['passwordChanged'] === 'true');
+						if ($shareType === OCP\Share::SHARE_TYPE_LINK && $shareWith['password'] === '') {
+							$shareWith = null;
+						} else {
+							$shareWith = $shareWith['password'];
+						}
+					} else {
+						/*
+						 * We need this branch since the calendar and contacts also use this
+						 * endpoint
+						 */
+						if ($shareType === OCP\Share::SHARE_TYPE_LINK && $shareWith === '') {
+							$shareWith = null;
+						}
 					}
+
  					$itemSourceName=(isset($_POST['itemSourceName'])) ? (string)$_POST['itemSourceName']:'';
 
 					$token = OCP\Share::shareItem(
@@ -59,16 +79,15 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 						$shareWith,
 						$_POST['permissions'],
 						$itemSourceName,
-						(!empty($_POST['expirationDate']) ? new \DateTime((string)$_POST['expirationDate']) : null)
+						(!empty($_POST['expirationDate']) ? new \DateTime((string)$_POST['expirationDate']) : null),
+						$passwordChanged
 					);
 
-					$response = [];
 					if (is_string($token)) {
-						$item = OCP\Share::getShareByToken($token, false);
-						$response['data'] = array('token' => $token,
-							'expiration' => $item['expiration']);
+						OC_JSON::success(array('data' => array('token' => $token)));
+					} else {
+						OC_JSON::success();
 					}
-					OC_JSON::success($response);
 				} catch (\OC\HintException $exception) {
 					OC_JSON::error(array('data' => array('message' => $exception->getHint())));
 				} catch (Exception $exception) {
@@ -185,6 +204,37 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 
 			$result = $mailNotification->sendLinkShareMail($to_address, $file, $link, $expiration);
 			if(empty($result)) {
+				// Get the token from the link
+				$linkParts = explode('/', $link);
+				$token = array_pop($linkParts);
+
+				// Get the share for the token
+				$share = \OCP\Share::getShareByToken($token, false);
+				if ($share !== false) {
+					$currentUser = \OC::$server->getUserSession()->getUser()->getUID();
+					$file = '/' . ltrim($file, '/');
+
+					// Check whether share belongs to the user and whether the file is the same
+					if ($share['file_target'] === $file && $share['uid_owner'] === $currentUser) {
+
+						// Get the path for the user
+						$view = new \OC\Files\View('/' . $currentUser . '/files');
+						$fileId = (int) $share['item_source'];
+						$path = $view->getPath((int) $share['item_source']);
+
+						if ($path !== null) {
+							$event = \OC::$server->getActivityManager()->generateEvent();
+							$event->setApp(\OCA\Files_Sharing\Activity::FILES_SHARING_APP)
+								->setType(\OCA\Files_Sharing\Activity::TYPE_SHARED)
+								->setAuthor($currentUser)
+								->setAffectedUser($currentUser)
+								->setObject('files', $fileId, $path)
+								->setSubject(\OCA\Files_Sharing\Activity::SUBJECT_SHARED_EMAIL, [$path, $to_address]);
+							\OC::$server->getActivityManager()->publish($event);
+						}
+					}
+				}
+
 				\OCP\JSON::success();
 			} else {
 				$l = \OC::$server->getL10N('core');
@@ -224,13 +274,21 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 					$reshare = false;
 				}
 				if ($_GET['checkShares'] == 'true') {
-					$shares = OCP\Share::getItemShared(
+					$sharesTMP = OCP\Share::getItemShared(
 						(string)$_GET['itemType'],
 						(string)$_GET['itemSource'],
 						OCP\Share::FORMAT_NONE,
 						null,
 						true
 					);
+					$ids = [];
+					$shares = [];
+					foreach($sharesTMP as $share) {
+						if (!isset($ids[$share['id']])) {
+							$ids[$share['id']] = true;
+							$shares[] = $share;
+						}
+					}
 				} else {
 					$shares = false;
 				}
