@@ -6,16 +6,12 @@ OC.L10N.register(
     "Admin" : "အက်ဒမင်",
     "today" : "ယနေ့",
     "yesterday" : "မနေ့က",
-    "_%n day ago_::_%n days ago_" : [""],
     "last month" : "ပြီးခဲ့သောလ",
-    "_%n month ago_::_%n months ago_" : [""],
     "last year" : "မနှစ်က",
-    "_%n year ago_::_%n years ago_" : [""],
-    "_%n hour ago_::_%n hours ago_" : [""],
-    "_%n minute ago_::_%n minutes ago_" : [""],
     "seconds ago" : "စက္ကန့်အနည်းငယ်က",
     "web services under your control" : "သင်၏ထိန်းချုပ်မှု့အောက်တွင်ရှိသော Web services",
     "Authentication error" : "ခွင့်ပြုချက်မအောင်မြင်",
-    "Could not find category \"%s\"" : "\"%s\"ခေါင်းစဉ်ကို ရှာမတွေ့ပါ"
+    "Could not find category \"%s\"" : "\"%s\"ခေါင်းစဉ်ကို ရှာမတွေ့ပါ",
+    "Apps" : "Apps"
 },
 "nplurals=1; plural=0;");
