@@ -1,20 +1,39 @@
 <?php
 /**
- * Copyright (c) 2013 Robin Appelman <icewind@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 namespace OC\Connector\Sabre;
 
+use OC\Connector\Sabre\Exception\InvalidPath;
+use OC\Connector\Sabre\Exception\FileLocked;
 use OC\Files\FileInfo;
-use OC\Files\Filesystem;
 use OC\Files\Mount\MoveableMount;
 use OCP\Files\StorageInvalidException;
 use OCP\Files\StorageNotAvailableException;
+use OCP\Lock\LockedException;
 
-class ObjectTree extends \Sabre\DAV\ObjectTree {
+class ObjectTree extends \Sabre\DAV\Tree {
 
 	/**
 	 * @var \OC\Files\View
@@ -22,14 +41,12 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 	protected $fileView;
 
 	/**
-	 * @var \OC\Files\Mount\Manager
+	 * @var  \OCP\Files\Mount\IMountManager
 	 */
 	protected $mountManager;
 
 	/**
 	 * Creates the object
-	 *
-	 * This method expects the rootObject to be passed as a parameter
 	 */
 	public function __construct() {
 	}
@@ -37,14 +54,43 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 	/**
 	 * @param \Sabre\DAV\INode $rootNode
 	 * @param \OC\Files\View $view
-	 * @param \OC\Files\Mount\Manager $mountManager
+	 * @param  \OCP\Files\Mount\IMountManager $mountManager
 	 */
-	public function init(\Sabre\DAV\INode $rootNode, \OC\Files\View $view, \OC\Files\Mount\Manager $mountManager) {
+	public function init(\Sabre\DAV\INode $rootNode, \OC\Files\View $view, \OCP\Files\Mount\IMountManager $mountManager) {
 		$this->rootNode = $rootNode;
 		$this->fileView = $view;
 		$this->mountManager = $mountManager;
 	}
 
+	/**
+	 * If the given path is a chunked file name, converts it
+	 * to the real file name. Only applies if the OC-CHUNKED header
+	 * is present.
+	 *
+	 * @param string $path chunk file path to convert
+	 * 
+	 * @return string path to real file
+	 */
+	private function resolveChunkFile($path) {
+		if (isset($_SERVER['HTTP_OC_CHUNKED'])) {
+			// resolve to real file name to find the proper node
+			list($dir, $name) = \Sabre\HTTP\URLUtil::splitPath($path);
+			if ($dir == '/' || $dir == '.') {
+				$dir = '';
+			}
+
+			$info = \OC_FileChunking::decodeName($name);
+			// only replace path if it was really the chunked file
+			if (isset($info['transferid'])) {
+				// getNodePath is called for multiple nodes within a chunk
+				// upload call
+				$path = $dir . '/' . $info['name'];
+				$path = ltrim($path, '/');
+			}
+		}
+		return $path;
+	}
+
 	/**
 	 * Returns the INode object for the requested path
 	 *
@@ -59,6 +105,14 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 		}
 
 		$path = trim($path, '/');
+		if ($path) {
+			try {
+				$this->fileView->verifyPath($path, basename($path));
+			} catch (\OCP\Files\InvalidPathException $ex) {
+				throw new InvalidPath($ex->getMessage());
+			}
+		}
+
 		if (isset($this->cache[$path])) {
 			return $this->cache[$path];
 		}
@@ -86,13 +140,18 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 				$info = null;
 			}
 		} else {
+			// resolve chunk file name to real name, if applicable
+			$path = $this->resolveChunkFile($path);
+
 			// read from cache
 			try {
 				$info = $this->fileView->getFileInfo($path);
 			} catch (StorageNotAvailableException $e) {
 				throw new \Sabre\DAV\Exception\ServiceUnavailable('Storage not available');
-			} catch (StorageInvalidException $e){
+			} catch (StorageInvalidException $e) {
 				throw new \Sabre\DAV\Exception\NotFound('Storage ' . $path . ' is invalid');
+			} catch (LockedException $e) {
+				throw new \Sabre\DAV\Exception\Locked();
 			}
 		}
 
@@ -101,9 +160,9 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 		}
 
 		if ($info->getType() === 'dir') {
-			$node = new \OC_Connector_Sabre_Directory($this->fileView, $info);
+			$node = new \OC\Connector\Sabre\Directory($this->fileView, $info);
 		} else {
-			$node = new \OC_Connector_Sabre_File($this->fileView, $info);
+			$node = new \OC\Connector\Sabre\File($this->fileView, $info);
 		}
 
 		$this->cache[$path] = $node;
@@ -129,10 +188,10 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 		$targetNodeExists = $this->nodeExists($destinationPath);
 		$sourceNode = $this->getNodeForPath($sourcePath);
 		if ($sourceNode instanceof \Sabre\DAV\ICollection && $targetNodeExists) {
-			throw new \Sabre\DAV\Exception\Forbidden('Could not copy directory ' . $sourceNode . ', target exists');
+			throw new \Sabre\DAV\Exception\Forbidden('Could not copy directory ' . $sourceNode->getName() . ', target exists');
 		}
-		list($sourceDir,) = \Sabre\DAV\URLUtil::splitPath($sourcePath);
-		list($destinationDir,) = \Sabre\DAV\URLUtil::splitPath($destinationPath);
+		list($sourceDir,) = \Sabre\HTTP\URLUtil::splitPath($sourcePath);
+		list($destinationDir,) = \Sabre\HTTP\URLUtil::splitPath($destinationPath);
 
 		$isMovableMount = false;
 		$sourceMount = $this->mountManager->find($this->fileView->getAbsolutePath($sourcePath));
@@ -164,24 +223,22 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 			}
 
 			$fileName = basename($destinationPath);
-			if (!\OCP\Util::isValidFileName($fileName)) {
-				throw new \Sabre\DAV\Exception\BadRequest();
+			try {
+				$this->fileView->verifyPath($destinationDir, $fileName);
+			} catch (\OCP\Files\InvalidPathException $ex) {
+				throw new InvalidPath($ex->getMessage());
 			}
 
 			$renameOkay = $this->fileView->rename($sourcePath, $destinationPath);
 			if (!$renameOkay) {
 				throw new \Sabre\DAV\Exception\Forbidden('');
 			}
-		} catch (\OCP\Files\StorageNotAvailableException $e) {
+		} catch (StorageNotAvailableException $e) {
 			throw new \Sabre\DAV\Exception\ServiceUnavailable($e->getMessage());
+		} catch (LockedException $e) {
+			throw new FileLocked($e->getMessage(), $e->getCode(), $e);
 		}
 
-		// update properties
-		$query = \OC_DB::prepare('UPDATE `*PREFIX*properties` SET `propertypath` = ?'
-			. ' WHERE `userid` = ? AND `propertypath` = ?');
-		$query->execute(array(\OC\Files\Filesystem::normalizePath($destinationPath), \OC_User::getUser(),
-			\OC\Files\Filesystem::normalizePath($sourcePath)));
-
 		$this->markDirty($sourceDir);
 		$this->markDirty($destinationDir);
 
@@ -206,26 +263,22 @@ class ObjectTree extends \Sabre\DAV\ObjectTree {
 		// this will trigger existence check
 		$this->getNodeForPath($source);
 
+		list($destinationDir, $destinationName) = \Sabre\HTTP\URLUtil::splitPath($destination);
 		try {
-			if ($this->fileView->is_file($source)) {
-				$this->fileView->copy($source, $destination);
-			} else {
-				$this->fileView->mkdir($destination);
-				$dh = $this->fileView->opendir($source);
-				if (is_resource($dh)) {
-					while (($subNode = readdir($dh)) !== false) {
-
-						if ($subNode == '.' || $subNode == '..') continue;
-						$this->copy($source . '/' . $subNode, $destination . '/' . $subNode);
-
-					}
-				}
-			}
-		} catch (\OCP\Files\StorageNotAvailableException $e) {
-			throw new \Sabre\DAV\Exception\ServiceUnavailable($e->getMessage());
+			$this->fileView->verifyPath($destinationDir, $destinationName);
+		} catch (\OCP\Files\InvalidPathException $ex) {
+			throw new InvalidPath($ex->getMessage());
 		}
 
-		list($destinationDir,) = \Sabre\DAV\URLUtil::splitPath($destination);
+		try {
+			$this->fileView->copy($source, $destination);
+		} catch (StorageNotAvailableException $e) {
+			throw new \Sabre\DAV\Exception\ServiceUnavailable($e->getMessage());
+		} catch (LockedException $e) {
+			throw new FileLocked($e->getMessage(), $e->getCode(), $e);
+		}
+
+		list($destinationDir,) = \Sabre\HTTP\URLUtil::splitPath($destination);
 		$this->markDirty($destinationDir);
 	}
 }
