@@ -1,33 +1,91 @@
 <?php
 /**
- * Copyright (c) 2012 Robin Appelman <icewind@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Carlos Cerrillo <ccerrillo@gmail.com>
+ * @author Felix Moeller <mail@felixmoeller.de>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Michael Gapczynski <GapczynskiM@gmail.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Philipp Kapfer <philipp.kapfer@gmx.at>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Scrutinizer Auto-Fixer <auto-fixer@scrutinizer-ci.com>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 namespace OC\Files\Storage;
 
+use Exception;
+use OC\Files\Filesystem;
+use OC\Files\Stream\Close;
+use Icewind\Streams\IteratorDirectory;
+use OC\MemCache\ArrayCache;
+use OCP\AppFramework\Http;
+use OCP\Constants;
+use OCP\Files;
+use OCP\Files\FileInfo;
 use OCP\Files\StorageInvalidException;
 use OCP\Files\StorageNotAvailableException;
-use Sabre\DAV\Exception;
+use OCP\Util;
+use Sabre\DAV\Client;
+use Sabre\DAV\Exception\NotFound;
+use Sabre\HTTP\ClientException;
+use Sabre\HTTP\ClientHttpException;
 
-class DAV extends \OC\Files\Storage\Common {
+/**
+ * Class DAV
+ *
+ * @package OC\Files\Storage
+ */
+class DAV extends Common {
+	/** @var string */
 	protected $password;
+	/** @var string */
 	protected $user;
+	/** @var string */
 	protected $host;
+	/** @var bool */
 	protected $secure;
+	/** @var string */
 	protected $root;
+	/** @var string */
 	protected $certPath;
+	/** @var bool */
 	protected $ready;
-	/**
-	 * @var \Sabre\DAV\Client
-	 */
+	/** @var Client */
 	private $client;
+	/** @var ArrayCache */
+	private $statCache;
+	/** @var array */
+	private static $tempFiles = [];
+	/** @var \OCP\Http\Client\IClientService */
+	private $httpClientService;
 
-	private static $tempFiles = array();
-
+	/**
+	 * @param array $params
+	 * @throws \Exception
+	 */
 	public function __construct($params) {
+		$this->statCache = new ArrayCache();
+		$this->httpClientService = \OC::$server->getHTTPClientService();
 		if (isset($params['host']) && isset($params['user']) && isset($params['password'])) {
 			$host = $params['host'];
 			//remove leading http[s], will be generated in createBaseUri()
@@ -75,17 +133,27 @@ class DAV extends \OC\Files\Storage\Common {
 			'password' => $this->password,
 		);
 
-		$this->client = new \Sabre\DAV\Client($settings);
+		$this->client = new Client($settings);
+		$this->client->setThrowExceptions(true);
 
 		if ($this->secure === true && $this->certPath) {
 			$this->client->addTrustedCertificates($this->certPath);
 		}
 	}
 
+	/**
+	 * Clear the stat cache
+	 */
+	public function clearStatCache() {
+		$this->statCache->clear();
+	}
+
+	/** {@inheritdoc} */
 	public function getId() {
 		return 'webdav::' . $this->user . '@' . $this->host . '/' . $this->root;
 	}
 
+	/** {@inheritdoc} */
 	public function createBaseUri() {
 		$baseUri = 'http';
 		if ($this->secure) {
@@ -95,80 +163,173 @@ class DAV extends \OC\Files\Storage\Common {
 		return $baseUri;
 	}
 
+	/** {@inheritdoc} */
 	public function mkdir($path) {
 		$this->init();
 		$path = $this->cleanPath($path);
-		return $this->simpleResponse('MKCOL', $path, null, 201);
+		$result = $this->simpleResponse('MKCOL', $path, null, 201);
+		if ($result) {
+			$this->statCache->set($path, true);
+		}
+		return $result;
 	}
 
+	/** {@inheritdoc} */
 	public function rmdir($path) {
 		$this->init();
-		$path = $this->cleanPath($path) . '/';
+		$path = $this->cleanPath($path);
 		// FIXME: some WebDAV impl return 403 when trying to DELETE
 		// a non-empty folder
-		return $this->simpleResponse('DELETE', $path, null, 204);
+		$result = $this->simpleResponse('DELETE', $path . '/', null, 204);
+		$this->statCache->clear($path . '/');
+		$this->statCache->remove($path);
+		return $result;
 	}
 
+	/** {@inheritdoc} */
 	public function opendir($path) {
 		$this->init();
 		$path = $this->cleanPath($path);
 		try {
-			$response = $this->client->propfind($this->encodePath($path), array(), 1);
+			$response = $this->client->propfind(
+				$this->encodePath($path),
+				array(),
+				1
+			);
 			$id = md5('webdav' . $this->root . $path);
 			$content = array();
 			$files = array_keys($response);
 			array_shift($files); //the first entry is the current directory
+
+			if (!$this->statCache->hasKey($path)) {
+				$this->statCache->set($path, true);
+			}
 			foreach ($files as $file) {
-				$file = urldecode(basename($file));
+				$file = urldecode($file);
+				// do not store the real entry, we might not have all properties
+				if (!$this->statCache->hasKey($path)) {
+					$this->statCache->set($file, true);
+				}
+				$file = basename($file);
 				$content[] = $file;
 			}
-			\OC\Files\Stream\Dir::register($id, $content);
-			return opendir('fakedir://' . $id);
-		} catch (Exception\NotFound $e) {
-			return false;
+			return IteratorDirectory::wrap($content);
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404) {
+				$this->statCache->clear($path . '/');
+				$this->statCache->set($path, false);
+				return false;
+			}
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return false;
 	}
 
-	public function filetype($path) {
-		$this->init();
+	/**
+	 * Propfind call with cache handling.
+	 *
+	 * First checks if information is cached.
+	 * If not, request it from the server then store to cache.
+	 *
+	 * @param string $path path to propfind
+	 *
+	 * @return array propfind response
+	 *
+	 * @throws NotFound
+	 */
+	private function propfind($path) {
 		$path = $this->cleanPath($path);
+		$cachedResponse = $this->statCache->get($path);
+		if ($cachedResponse === false) {
+			// we know it didn't exist
+			throw new NotFound();
+		}
+		// we either don't know it, or we know it exists but need more details
+		if (is_null($cachedResponse) || $cachedResponse === true) {
+			$this->init();
+			try {
+				$response = $this->client->propfind(
+					$this->encodePath($path),
+					array(
+						'{DAV:}getlastmodified',
+						'{DAV:}getcontentlength',
+						'{DAV:}getcontenttype',
+						'{http://owncloud.org/ns}permissions',
+						'{DAV:}resourcetype',
+						'{DAV:}getetag',
+					)
+				);
+				$this->statCache->set($path, $response);
+			} catch (NotFound $e) {
+				// remember that this path did not exist
+				$this->statCache->clear($path . '/');
+				$this->statCache->set($path, false);
+				throw $e;
+			}
+		} else {
+			$response = $cachedResponse;
+		}
+		return $response;
+	}
+
+	/** {@inheritdoc} */
+	public function filetype($path) {
 		try {
-			$response = $this->client->propfind($this->encodePath($path), array('{DAV:}resourcetype'));
+			$response = $this->propfind($path);
 			$responseType = array();
 			if (isset($response["{DAV:}resourcetype"])) {
 				$responseType = $response["{DAV:}resourcetype"]->resourceType;
 			}
 			return (count($responseType) > 0 and $responseType[0] == "{DAV:}collection") ? 'dir' : 'file';
-		} catch (Exception\NotFound $e) {
-			return false;
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404) {
+				return false;
+			}
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return false;
 	}
 
+	/** {@inheritdoc} */
 	public function file_exists($path) {
-		$this->init();
-		$path = $this->cleanPath($path);
 		try {
-			$this->client->propfind($this->encodePath($path), array('{DAV:}resourcetype'));
+			$path = $this->cleanPath($path);
+			$cachedState = $this->statCache->get($path);
+			if ($cachedState === false) {
+				// we know the file doesn't exist
+				return false;
+			} else if (!is_null($cachedState)) {
+				return true;
+			}
+			// need to get from server
+			$this->propfind($path);
 			return true; //no 404 exception
-		} catch (Exception\NotFound $e) {
-			return false;
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404) {
+				return false;
+			}
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return false;
 	}
 
+	/** {@inheritdoc} */
 	public function unlink($path) {
 		$this->init();
-		return $this->simpleResponse('DELETE', $path, null, 204);
+		$path = $this->cleanPath($path);
+		$result = $this->simpleResponse('DELETE', $path, null, 204);
+		$this->statCache->clear($path . '/');
+		$this->statCache->remove($path);
+		return $result;
 	}
 
+	/** {@inheritdoc} */
 	public function fopen($path, $mode) {
 		$this->init();
 		$path = $this->cleanPath($path);
@@ -178,35 +339,22 @@ class DAV extends \OC\Files\Storage\Common {
 				if (!$this->file_exists($path)) {
 					return false;
 				}
-				//straight up curl instead of sabredav here, sabredav put's the entire get result in memory
-				$curl = curl_init();
-				$fp = fopen('php://temp', 'r+');
-				curl_setopt($curl, CURLOPT_USERPWD, $this->user . ':' . $this->password);
-				curl_setopt($curl, CURLOPT_URL, $this->createBaseUri() . $this->encodePath($path));
-				curl_setopt($curl, CURLOPT_FILE, $fp);
-				curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
-				if(defined('CURLOPT_PROTOCOLS')) {
-					curl_setopt($curl, CURLOPT_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);
-				}
-				if(defined('CURLOPT_REDIR_PROTOCOLS')) {
-					curl_setopt($curl, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);
-				}
-				if ($this->secure === true) {
-					curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
-					curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
-					if ($this->certPath) {
-						curl_setopt($curl, CURLOPT_CAINFO, $this->certPath);
+				$response = $this->httpClientService
+					->newClient()
+					->get($this->createBaseUri() . $this->encodePath($path), [
+						'auth' => [$this->user, $this->password],
+						'stream' => true
+					]);
+
+				if ($response->getStatusCode() !== Http::STATUS_OK) {
+					if ($response->getStatusCode() === Http::STATUS_LOCKED) {
+						throw new \OCP\Lock\LockedException($path);
+					} else {
+						Util::writeLog("webdav client", 'Guzzle get returned status code ' . $response->getStatusCode(), Util::ERROR);
 					}
 				}
 
-				curl_exec($curl);
-				$statusCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
-				if ($statusCode !== 200) {
-					\OCP\Util::writeLog("webdav client", 'curl GET ' . curl_getinfo($curl, CURLINFO_EFFECTIVE_URL) . ' returned status code ' . $statusCode, \OCP\Util::ERROR);
-				}
-				curl_close($curl);
-				rewind($fp);
-				return $fp;
+				return $response->getBody();
 			case 'w':
 			case 'wb':
 			case 'a':
@@ -220,6 +368,7 @@ class DAV extends \OC\Files\Storage\Common {
 			case 'c':
 			case 'c+':
 				//emulate these
+				$tempManager = \OC::$server->getTempManager();
 				if (strrpos($path, '.') !== false) {
 					$ext = substr($path, strrpos($path, '.'));
 				} else {
@@ -229,19 +378,26 @@ class DAV extends \OC\Files\Storage\Common {
 					if (!$this->isUpdatable($path)) {
 						return false;
 					}
-					$tmpFile = $this->getCachedFile($path);
+					if ($mode === 'w' or $mode === 'w+') {
+						$tmpFile = $tempManager->getTemporaryFile($ext);
+					} else {
+						$tmpFile = $this->getCachedFile($path);
+					}
 				} else {
 					if (!$this->isCreatable(dirname($path))) {
 						return false;
 					}
-					$tmpFile = \OCP\Files::tmpFile($ext);
+					$tmpFile = $tempManager->getTemporaryFile($ext);
 				}
-				\OC\Files\Stream\Close::registerCallback($tmpFile, array($this, 'writeBack'));
+				Close::registerCallback($tmpFile, array($this, 'writeBack'));
 				self::$tempFiles[$tmpFile] = $path;
 				return fopen('close://' . $tmpFile, $mode);
 		}
 	}
 
+	/**
+	 * @param string $tmpFile
+	 */
 	public function writeBack($tmpFile) {
 		if (isset(self::$tempFiles[$tmpFile])) {
 			$this->uploadFile($tmpFile, self::$tempFiles[$tmpFile]);
@@ -249,21 +405,31 @@ class DAV extends \OC\Files\Storage\Common {
 		}
 	}
 
+	/** {@inheritdoc} */
 	public function free_space($path) {
 		$this->init();
 		$path = $this->cleanPath($path);
 		try {
+			// TODO: cacheable ?
 			$response = $this->client->propfind($this->encodePath($path), array('{DAV:}quota-available-bytes'));
 			if (isset($response['{DAV:}quota-available-bytes'])) {
-				return (int)$response['{DAV:}quota-available-bytes'];
+				$freeSpace = (int)$response['{DAV:}quota-available-bytes'];
+				if ($freeSpace === FileInfo::SPACE_UNLIMITED) {
+					// most of the code cannot cope with unlimited storage,
+					// so as a workaround convert to SPACE_UNKNOWN which is a
+					// value recognized in many places
+					return FileInfo::SPACE_UNKNOWN;
+				}
+				return $freeSpace;
 			} else {
-				return \OCP\Files\FileInfo::SPACE_UNKNOWN;
+				return FileInfo::SPACE_UNKNOWN;
 			}
 		} catch (\Exception $e) {
-			return \OCP\Files\FileInfo::SPACE_UNKNOWN;
+			return FileInfo::SPACE_UNKNOWN;
 		}
 	}
 
+	/** {@inheritdoc} */
 	public function touch($path, $mtime = null) {
 		$this->init();
 		if (is_null($mtime)) {
@@ -274,11 +440,16 @@ class DAV extends \OC\Files\Storage\Common {
 		// if file exists, update the mtime, else create a new empty file
 		if ($this->file_exists($path)) {
 			try {
+				$this->statCache->remove($path);
 				$this->client->proppatch($this->encodePath($path), array('{DAV:}lastmodified' => $mtime));
-			} catch (Exception\NotImplemented $e) {
+			} catch (ClientHttpException $e) {
+				if ($e->getHttpStatus() === 501) {
+					return false;
+				}
+				$this->convertException($e, $path);
 				return false;
 			} catch (\Exception $e) {
-				$this->convertException($e);
+				$this->convertException($e, $path);
 				return false;
 			}
 		} else {
@@ -287,47 +458,58 @@ class DAV extends \OC\Files\Storage\Common {
 		return true;
 	}
 
+	/**
+	 * @param string $path
+	 * @param string $data
+	 * @return int
+	 */
+	public function file_put_contents($path, $data) {
+		$path = $this->cleanPath($path);
+		$result = parent::file_put_contents($path, $data);
+		$this->statCache->remove($path);
+		return $result;
+	}
+
+	/**
+	 * @param string $path
+	 * @param string $target
+	 */
 	protected function uploadFile($path, $target) {
 		$this->init();
+
+		// invalidate
+		$target = $this->cleanPath($target);
+		$this->statCache->remove($target);
 		$source = fopen($path, 'r');
 
-		$curl = curl_init();
-		curl_setopt($curl, CURLOPT_USERPWD, $this->user . ':' . $this->password);
-		curl_setopt($curl, CURLOPT_URL, $this->createBaseUri() . $this->encodePath($target));
-		curl_setopt($curl, CURLOPT_BINARYTRANSFER, true);
-		curl_setopt($curl, CURLOPT_INFILE, $source); // file pointer
-		curl_setopt($curl, CURLOPT_INFILESIZE, filesize($path));
-		curl_setopt($curl, CURLOPT_PUT, true);
-		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
-		if(defined('CURLOPT_PROTOCOLS')) {
-			curl_setopt($curl, CURLOPT_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);
-		}
-		if(defined('CURLOPT_REDIR_PROTOCOLS')) {
-			curl_setopt($curl, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);
-		}
-		if ($this->secure === true) {
-			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
-			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
-			if ($this->certPath) {
-				curl_setopt($curl, CURLOPT_CAINFO, $this->certPath);
-			}
-		}
-		curl_exec($curl);
-		$statusCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
-		if ($statusCode !== 200) {
-			\OCP\Util::writeLog("webdav client", 'curl GET ' . curl_getinfo($curl, CURLINFO_EFFECTIVE_URL) . ' returned status code ' . $statusCode, \OCP\Util::ERROR);
-		}
-		curl_close($curl);
-		fclose($source);
+		$this->httpClientService
+			->newClient()
+			->put($this->createBaseUri() . $this->encodePath($target), [
+				'body' => $source,
+				'auth' => [$this->user, $this->password]
+			]);
+
 		$this->removeCachedFile($target);
 	}
 
+	/** {@inheritdoc} */
 	public function rename($path1, $path2) {
 		$this->init();
-		$path1 = $this->encodePath($this->cleanPath($path1));
-		$path2 = $this->createBaseUri() . $this->encodePath($this->cleanPath($path2));
+		$path1 = $this->cleanPath($path1);
+		$path2 = $this->cleanPath($path2);
 		try {
-			$this->client->request('MOVE', $path1, null, array('Destination' => $path2));
+			$this->client->request(
+				'MOVE',
+				$this->encodePath($path1),
+				null,
+				array(
+					'Destination' => $this->createBaseUri() . $this->encodePath($path2)
+				)
+			);
+			$this->statCache->clear($path1 . '/');
+			$this->statCache->clear($path2 . '/');
+			$this->statCache->set($path1, false);
+			$this->statCache->set($path2, true);
 			$this->removeCachedFile($path1);
 			$this->removeCachedFile($path2);
 			return true;
@@ -337,12 +519,15 @@ class DAV extends \OC\Files\Storage\Common {
 		return false;
 	}
 
+	/** {@inheritdoc} */
 	public function copy($path1, $path2) {
 		$this->init();
 		$path1 = $this->encodePath($this->cleanPath($path1));
 		$path2 = $this->createBaseUri() . $this->encodePath($this->cleanPath($path2));
 		try {
 			$this->client->request('COPY', $path1, null, array('Destination' => $path2));
+			$this->statCache->clear($path2 . '/');
+			$this->statCache->set($path2, true);
 			$this->removeCachedFile($path2);
 			return true;
 		} catch (\Exception $e) {
@@ -351,28 +536,29 @@ class DAV extends \OC\Files\Storage\Common {
 		return false;
 	}
 
+	/** {@inheritdoc} */
 	public function stat($path) {
-		$this->init();
-		$path = $this->cleanPath($path);
 		try {
-			$response = $this->client->propfind($this->encodePath($path), array('{DAV:}getlastmodified', '{DAV:}getcontentlength'));
+			$response = $this->propfind($path);
 			return array(
 				'mtime' => strtotime($response['{DAV:}getlastmodified']),
 				'size' => (int)isset($response['{DAV:}getcontentlength']) ? $response['{DAV:}getcontentlength'] : 0,
 			);
-		} catch (Exception\NotFound $e) {
-			return array();
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404) {
+				return array();
+			}
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return array();
 	}
 
+	/** {@inheritdoc} */
 	public function getMimeType($path) {
-		$this->init();
-		$path = $this->cleanPath($path);
 		try {
-			$response = $this->client->propfind($this->encodePath($path), array('{DAV:}getcontenttype', '{DAV:}resourcetype'));
+			$response = $this->propfind($path);
 			$responseType = array();
 			if (isset($response["{DAV:}resourcetype"])) {
 				$responseType = $response["{DAV:}resourcetype"]->resourceType;
@@ -385,22 +571,26 @@ class DAV extends \OC\Files\Storage\Common {
 			} else {
 				return false;
 			}
-		} catch (Exception\NotFound $e) {
-			return false;
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404) {
+				return false;
+			}
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return false;
 	}
 
 	/**
 	 * @param string $path
+	 * @return string
 	 */
 	public function cleanPath($path) {
-		if ($path === "") {
+		if ($path === '') {
 			return $path;
 		}
-		$path = \OC\Files\Filesystem::normalizePath($path);
+		$path = Filesystem::normalizePath($path);
 		// remove leading slash
 		return substr($path, 1);
 	}
@@ -419,21 +609,27 @@ class DAV extends \OC\Files\Storage\Common {
 	/**
 	 * @param string $method
 	 * @param string $path
-	 * @param integer $expected
+	 * @param string|resource|null $body
+	 * @param int $expected
+	 * @return bool
+	 * @throws StorageInvalidException
+	 * @throws StorageNotAvailableException
 	 */
 	private function simpleResponse($method, $path, $body, $expected) {
 		$path = $this->cleanPath($path);
 		try {
 			$response = $this->client->request($method, $this->encodePath($path), $body);
 			return $response['statusCode'] == $expected;
-		} catch (Exception\NotFound $e) {
-			if ($method === 'DELETE') {
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404 && $method === 'DELETE') {
+				$this->statCache->clear($path . '/');
+				$this->statCache->set($path, false);
 				return false;
 			}
 
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 		}
 		return false;
 	}
@@ -442,39 +638,40 @@ class DAV extends \OC\Files\Storage\Common {
 	 * check if curl is installed
 	 */
 	public static function checkDependencies() {
-		if (function_exists('curl_init')) {
-			return true;
-		} else {
-			return array('curl');
-		}
+		return true;
 	}
 
+	/** {@inheritdoc} */
 	public function isUpdatable($path) {
-		return (bool)($this->getPermissions($path) & \OCP\Constants::PERMISSION_UPDATE);
+		return (bool)($this->getPermissions($path) & Constants::PERMISSION_UPDATE);
 	}
 
+	/** {@inheritdoc} */
 	public function isCreatable($path) {
-		return (bool)($this->getPermissions($path) & \OCP\Constants::PERMISSION_CREATE);
+		return (bool)($this->getPermissions($path) & Constants::PERMISSION_CREATE);
 	}
 
+	/** {@inheritdoc} */
 	public function isSharable($path) {
-		return (bool)($this->getPermissions($path) & \OCP\Constants::PERMISSION_SHARE);
+		return (bool)($this->getPermissions($path) & Constants::PERMISSION_SHARE);
 	}
 
+	/** {@inheritdoc} */
 	public function isDeletable($path) {
-		return (bool)($this->getPermissions($path) & \OCP\Constants::PERMISSION_DELETE);
+		return (bool)($this->getPermissions($path) & Constants::PERMISSION_DELETE);
 	}
 
+	/** {@inheritdoc} */
 	public function getPermissions($path) {
 		$this->init();
 		$path = $this->cleanPath($path);
-		$response = $this->client->propfind($this->encodePath($path), array('{http://owncloud.org/ns}permissions'));
+		$response = $this->propfind($path);
 		if (isset($response['{http://owncloud.org/ns}permissions'])) {
 			return $this->parsePermissions($response['{http://owncloud.org/ns}permissions']);
 		} else if ($this->is_dir($path)) {
-			return \OCP\Constants::PERMISSION_ALL;
+			return Constants::PERMISSION_ALL;
 		} else if ($this->file_exists($path)) {
-			return \OCP\Constants::PERMISSION_ALL - \OCP\Constants::PERMISSION_CREATE;
+			return Constants::PERMISSION_ALL - Constants::PERMISSION_CREATE;
 		} else {
 			return 0;
 		}
@@ -484,7 +681,7 @@ class DAV extends \OC\Files\Storage\Common {
 	public function getETag($path) {
 		$this->init();
 		$path = $this->cleanPath($path);
-		$response = $this->client->propfind($this->encodePath($path), array('{DAV:}getetag'));
+		$response = $this->propfind($path);
 		if (isset($response['{DAV:}getetag'])) {
 			return trim($response['{DAV:}getetag'], '"');
 		}
@@ -496,19 +693,19 @@ class DAV extends \OC\Files\Storage\Common {
 	 * @return int
 	 */
 	protected function parsePermissions($permissionsString) {
-		$permissions = \OCP\Constants::PERMISSION_READ;
+		$permissions = Constants::PERMISSION_READ;
 		if (strpos($permissionsString, 'R') !== false) {
-			$permissions |= \OCP\Constants::PERMISSION_SHARE;
+			$permissions |= Constants::PERMISSION_SHARE;
 		}
 		if (strpos($permissionsString, 'D') !== false) {
-			$permissions |= \OCP\Constants::PERMISSION_DELETE;
+			$permissions |= Constants::PERMISSION_DELETE;
 		}
 		if (strpos($permissionsString, 'W') !== false) {
-			$permissions |= \OCP\Constants::PERMISSION_UPDATE;
+			$permissions |= Constants::PERMISSION_UPDATE;
 		}
 		if (strpos($permissionsString, 'CK') !== false) {
-			$permissions |= \OCP\Constants::PERMISSION_CREATE;
-			$permissions |= \OCP\Constants::PERMISSION_UPDATE;
+			$permissions |= Constants::PERMISSION_CREATE;
+			$permissions |= Constants::PERMISSION_UPDATE;
 		}
 		return $permissions;
 	}
@@ -525,11 +722,9 @@ class DAV extends \OC\Files\Storage\Common {
 		$this->init();
 		$path = $this->cleanPath($path);
 		try {
-			$response = $this->client->propfind($this->encodePath($path), array(
-				'{DAV:}getlastmodified',
-				'{DAV:}getetag',
-				'{http://owncloud.org/ns}permissions'
-			));
+			// force refresh for $path
+			$this->statCache->remove($path);
+			$response = $this->propfind($path);
 			if (isset($response['{DAV:}getetag'])) {
 				$cachedData = $this->getCache()->get($path);
 				$etag = null;
@@ -548,10 +743,18 @@ class DAV extends \OC\Files\Storage\Common {
 				$remoteMtime = strtotime($response['{DAV:}getlastmodified']);
 				return $remoteMtime > $time;
 			}
-		} catch (Exception\NotFound $e) {
+		} catch (ClientHttpException $e) {
+			if ($e->getHttpStatus() === 404 || $e->getHttpStatus() === 405) {
+				if ($path === '') {
+					// if root is gone it means the storage is not available
+					throw new StorageNotAvailableException(get_class($e) . ': ' . $e->getMessage());
+				}
+				return false;
+			}
+			$this->convertException($e, $path);
 			return false;
 		} catch (\Exception $e) {
-			$this->convertException($e);
+			$this->convertException($e, $path);
 			return false;
 		}
 	}
@@ -562,27 +765,35 @@ class DAV extends \OC\Files\Storage\Common {
 	 * This will either throw StorageInvalidException, StorageNotAvailableException
 	 * or do nothing.
 	 *
-	 * @param \Sabre\Dav\Exception $e sabre exception
+	 * @param Exception $e sabre exception
+	 * @param string $path optional path from the operation
 	 *
 	 * @throws StorageInvalidException if the storage is invalid, for example
 	 * when the authentication expired or is invalid
 	 * @throws StorageNotAvailableException if the storage is not available,
 	 * which might be temporary
 	 */
-	private function convertException(\Exception $e) {
-		\OCP\Util::writeLog('files_external', $e->getMessage(), \OCP\Util::ERROR);
-		if ($e instanceof \Sabre\DAV\Exception\NotAuthenticated) {
-			// either password was changed or was invalid all along
-			throw new StorageInvalidException(get_class($e).': '.$e->getMessage());
-		} else if ($e instanceof \Sabre\DAV\Exception\MethodNotAllowed) {
-			// ignore exception, false will be returned
-			return;
-		} else if ($e instanceof \Sabre\Dav\Exception) {
-			throw new StorageNotAvailableException(get_class($e).': '.$e->getMessage());
+	private function convertException(Exception $e, $path = '') {
+		Util::writeLog('files_external', $e->getMessage(), Util::ERROR);
+		if ($e instanceof ClientHttpException) {
+			if ($e->getHttpStatus() === 423) {
+				throw new \OCP\Lock\LockedException($path);
+			}
+			if ($e->getHttpStatus() === 401) {
+				// either password was changed or was invalid all along
+				throw new StorageInvalidException(get_class($e) . ': ' . $e->getMessage());
+			} else if ($e->getHttpStatus() === 405) {
+				// ignore exception for MethodNotAllowed, false will be returned
+				return;
+			}
+			throw new StorageNotAvailableException(get_class($e) . ': ' . $e->getMessage());
+		} else if ($e instanceof ClientException) {
+			// connection timeout or refused, server could be temporarily down
+			throw new StorageNotAvailableException(get_class($e) . ': ' . $e->getMessage());
 		} else if ($e instanceof \InvalidArgumentException) {
 			// parse error because the server returned HTML instead of XML,
 			// possibly temporarily down
-			throw new StorageNotAvailableException(get_class($e).': '.$e->getMessage());
+			throw new StorageNotAvailableException(get_class($e) . ': ' . $e->getMessage());
 		} else if (($e instanceof StorageNotAvailableException) || ($e instanceof StorageInvalidException)) {
 			// rethrow
 			throw $e;
