@@ -4,6 +4,7 @@
  * @author Morris Jobke <hey@morrisjobke.de>
  * @author Normal Ra <normalraw@gmail.com>
  * @author Olivier Paroz <github@oparoz.com>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
  * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  * @author Vincent Petry <pvince81@owncloud.com>
  *
@@ -29,11 +30,23 @@ namespace OC\Repair;
 use OC\Hooks\BasicEmitter;
 
 class RepairMimeTypes extends BasicEmitter implements \OC\RepairStep {
+	/**
+	 * @var \OCP\IConfig
+	 */
+	protected $config;
+
 	/**
 	 * @var int
 	 */
 	protected $folderMimeTypeId;
 
+	/**
+	 * @param \OCP\IConfig $config
+	 */
+	public function __construct($config) {
+		$this->config = $config;
+	}
+
 	public function getName() {
 		return 'Repair mime types';
 	}
@@ -247,40 +260,118 @@ class RepairMimeTypes extends BasicEmitter implements \OC\RepairStep {
 		self::updateMimetypes($updatedMimetypes);
 	}
 
+	private function introduceJavaMimeType() {
+		$updatedMimetypes = array(
+			'class' => 'application/java',
+			'java' => 'text/x-java-source',
+		);
+
+		self::updateMimetypes($updatedMimetypes);
+	}
+
+	private function introduceHppMimeType() {
+		$updatedMimetypes = array(
+			'hpp' => 'text/x-h',
+		);
+
+		self::updateMimetypes($updatedMimetypes);
+	}
+
+	private function introduceRssMimeType() {
+		$updatedMimetypes = array(
+			'rss' => 'application/rss+xml',
+		);
+
+		self::updateMimetypes($updatedMimetypes);
+	}
+
+	private function introduceRtfMimeType() {
+		$updatedMimetypes = array(
+			'rtf' => 'text/rtf',
+		);
+
+		self::updateMimetypes($updatedMimetypes);
+	}
+
+	private function introduceRichDocumentsMimeTypes() {
+		$updatedMimetypes = array(
+			'lwp' => 'application/vnd.lotus-wordpro',
+			'one' => 'application/msonenote',
+			'vsd' => 'application/vnd.visio',
+			'wpd' => 'application/vnd.wordperfect',
+		);
+
+		self::updateMimetypes($updatedMimetypes);
+	}
+
 	/**
 	 * Fix mime types
 	 */
 	public function run() {
-		if ($this->fixOfficeMimeTypes()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed office mime types'));
+
+		$ocVersionFromBeforeUpdate = $this->config->getSystemValue('version', '0.0.0');
+
+		// NOTE TO DEVELOPERS: when adding new mime types, please make sure to
+		// add a version comparison to avoid doing it every time
+
+		// only update mime types if necessary as it can be expensive
+		if (version_compare($ocVersionFromBeforeUpdate, '8.2.0', '<')) {
+			if ($this->fixOfficeMimeTypes()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed office mime types'));
+			}
+
+			if ($this->fixApkMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed APK mime type'));
+			}
+
+			if ($this->fixFontsMimeTypes()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed fonts mime types'));
+			}
+
+			if ($this->fixPostscriptMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed Postscript mime types'));
+			}
+
+			if ($this->introduceRawMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed Raw mime types'));
+			}
+
+			if ($this->introduce3dImagesMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed 3D images mime types'));
+			}
+
+			if ($this->introduceConfMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed Conf/cnf mime types'));
+			}
+
+			if ($this->introduceYamlMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed Yaml/Yml mime types'));
+			}
 		}
 
-		if ($this->fixApkMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed APK mime type'));
+		// Mimetype updates from #19272
+		if (version_compare($ocVersionFromBeforeUpdate, '8.2.0.8', '<')) {
+			if ($this->introduceJavaMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed java/class mime types'));
+			}
+
+			if ($this->introduceHppMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed hpp mime type'));
+			}
+
+			if ($this->introduceRssMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed rss mime type'));
+			}
+
+			if ($this->introduceRtfMimeType()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed rtf mime type'));
+			}
 		}
 
-		if ($this->fixFontsMimeTypes()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed fonts mime types'));
-		}
-
-		if ($this->fixPostscriptMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed Postscript mime types'));
-		}
-
-		if ($this->introduceRawMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed Raw mime types'));
-		}
-
-		if ($this->introduce3dImagesMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed 3D images mime types'));
-		}
-
-		if ($this->introduceConfMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed Conf/cnf mime types'));
-		}
-
-		if ($this->introduceYamlMimeType()) {
-			$this->emit('\OC\Repair', 'info', array('Fixed Yaml/Yml mime types'));
+		if (version_compare($ocVersionFromBeforeUpdate, '8.2.2.3', '<')) {
+			if ($this->introduceRichDocumentsMimeTypes()) {
+				$this->emit('\OC\Repair', 'info', array('Fixed richdocuments additional office mime types'));
+			}
 		}
 	}
 }
