@@ -1,23 +1,15 @@
 OC.L10N.register(
     "settings",
     {
-    "Security & Setup Warnings" : "Säkerhets & Inställningsvarningar",
-    "Cron" : "Cron",
     "Sharing" : "Dela",
-    "Security" : "Säkerhet",
-    "Email Server" : "E-postserver",
+    "External Storage" : "Extern lagring",
+    "Cron" : "Cron",
     "Log" : "Logg",
+    "Updates" : "Uppdateringar",
     "Authentication error" : "Fel vid autentisering",
     "Your full name has been changed." : "Hela ditt namn har ändrats",
     "Unable to change full name" : "Kunde inte ändra hela namnet",
-    "Files decrypted successfully" : "Filerna dekrypterades utan fel",
-    "Couldn't decrypt your files, please check your owncloud.log or ask your administrator" : "Det gick inte att dekryptera dina filer, kontrollera din owncloud.log eller fråga administratören",
-    "Couldn't decrypt your files, check your password and try again" : "Det gick inte att dekryptera filerna, kontrollera ditt lösenord och försök igen",
-    "Encryption keys deleted permanently" : "Krypteringsnycklar raderades permanent",
-    "Couldn't permanently delete your encryption keys, please check your owncloud.log or ask your administrator" : "Det gick inte att permanent ta bort dina krypteringsnycklar, kontrollera din owncloud.log eller fråga din administratör",
     "Couldn't remove app." : "Kunde inte ta bort applikationen.",
-    "Backups restored successfully" : "Återställning av säkerhetskopior lyckades",
-    "Couldn't restore your encryption keys, please check your owncloud.log or ask your administrator" : "Kan inte återställa dina krypteringsnycklar, vänligen kontrollera din owncloud.log eller fråga din administratör.",
     "Language changed" : "Språk ändrades",
     "Invalid request" : "Ogiltig begäran",
     "Admins can't remove themself from the admin group" : "Administratörer kan inte ta bort sig själva från admingruppen",
@@ -28,19 +20,15 @@ OC.L10N.register(
     "No user supplied" : "Ingen användare angiven",
     "Please provide an admin recovery password, otherwise all user data will be lost" : "Ange ett återställningslösenord för administratören. Annars kommer all användardata förloras",
     "Wrong admin recovery password. Please check the password and try again." : "Felaktigt återställningslösenord för administratör. Kolla lösenordet och prova igen.",
-    "Back-end doesn't support password change, but the users encryption key was successfully updated." : "Gränssnittet stödjer inte byte av lösenord, men användarnas krypteringsnyckel blev uppdaterad.",
     "Unable to change password" : "Kunde inte ändra lösenord",
     "Enabled" : "Aktiverad",
     "Not enabled" : "Inte aktiverad",
-    "Recommended" : "Rekomenderad",
     "Group already exists." : "Gruppen finns redan.",
     "Unable to add group." : "Lyckades inte lägga till grupp.",
     "Unable to delete group." : "Lyckades inte radera grupp.",
     "log-level out of allowed range" : "logg-nivå utanför tillåtet område",
     "Saved" : "Sparad",
     "test email settings" : "testa e-post inställningar",
-    "If you received this email, the settings seem to be correct." : "Om du mottog detta e-postmeddelande, verkar dina inställningar vara korrekta.",
-    "A problem occurred while sending the email. Please revise your settings." : "Ett problem uppstod när e-postmeddelandet skickades. Vänligen se över dina inställningar.",
     "Email sent" : "E-post skickat",
     "You need to set your user email before being able to send test emails." : "Du behöver ställa in din användares e-postadress före du kan skicka test e-post.",
     "Invalid mail address" : "Ogiltig e-postadress",
@@ -55,6 +43,7 @@ OC.L10N.register(
     "Add trusted domain" : "Lägg till betrodd domän",
     "Sending..." : "Skickar ...",
     "All" : "Alla",
+    "Update to %s" : "Uppdatera till %s",
     "Please wait...." : "Var god vänta ...",
     "Error while disabling app" : "Fel vid inaktivering av app",
     "Disable" : "Deaktivera",
@@ -74,9 +63,6 @@ OC.L10N.register(
     "Strong password" : "Starkt lösenord",
     "Valid until {date}" : "Giltig t.o.m. {date}",
     "Delete" : "Radera",
-    "Decrypting files... Please wait, this can take some time." : "Dekrypterar filer ... Vänligen vänta, detta kan ta en stund.",
-    "Delete encryption keys permanently." : "Radera krypteringsnycklar permanent",
-    "Restore encryption keys." : "Återställ krypteringsnycklar",
     "Groups" : "Grupper",
     "Unable to delete {objName}" : "Kunde inte radera {objName}",
     "Error creating group" : "Fel vid skapande av grupp",
@@ -92,9 +78,7 @@ OC.L10N.register(
     "A valid password must be provided" : "Ett giltigt lösenord måste anges",
     "A valid email must be provided" : "En giltig e-postadress måste anges",
     "__language_name__" : "__language_name__",
-    "Personal Info" : "Personlig info",
     "SSL root certificates" : "SSL rotcertifikat",
-    "Encryption" : "Kryptering",
     "Everything (fatal issues, errors, warnings, info, debug)" : "Allting (allvarliga fel, fel, varningar, info, debug)",
     "Info, warnings, errors and fatal issues" : "Info, varningar och allvarliga fel",
     "Warnings, errors and fatal issues" : "Varningar, fel och allvarliga fel",
@@ -106,35 +90,15 @@ OC.L10N.register(
     "NT LAN Manager" : "NT LAN Manager",
     "SSL" : "SSL",
     "TLS" : "TLS",
-    "Security Warning" : "Säkerhetsvarning",
-    "You are accessing %s via HTTP. We strongly suggest you configure your server to require using HTTPS instead." : "Du ansluter till %s via HTTP. Vi rekommenderar starkt att du konfigurerar din server att använda HTTPS istället.",
-    "Read-Only config enabled" : "Skrivskyddad konfiguration påslagen",
     "The Read-Only config has been enabled. This prevents setting some configurations via the web-interface. Furthermore, the file needs to be made writable manually for every update." : "Lär-bara konfigureringen har blivit aktiv. Detta förhindrar att några konfigureringar kan sättas via web-gränssnittet.",
-    "Setup Warning" : "Installationsvarning",
     "PHP is apparently setup to strip inline doc blocks. This will make several core apps inaccessible." : "PHP är tydligen inställd för att rensa inline doc block. Detta kommer att göra flera kärnapplikationer otillgängliga.",
     "This is probably caused by a cache/accelerator such as Zend OPcache or eAccelerator." : "Detta orsakas troligtvis av en cache/accelerator som t ex Zend OPchache eller eAccelerator.",
-    "Database Performance Info" : "Databasprestanda Information",
-    "Microsoft Windows Platform" : "Microsoft Windows-platform",
     "Your server is running on Microsoft Windows. We highly recommend Linux for optimal user experience." : "Din server använder Microsoft Windows. Vi rekommenderar starkt Linux för en optimal användarerfarenhet.",
-    "Module 'fileinfo' missing" : "Modulen \"fileinfo\" saknas",
     "The PHP module 'fileinfo' is missing. We strongly recommend to enable this module to get best results with mime-type detection." : "PHP-modulen 'fileinfo' saknas. Vi rekommenderar starkt att aktivera den här modulen för att kunna upptäcka korrekt mime-typ.",
-    "PHP charset is not set to UTF-8" : "PHP-teckenuppsättning är inte satt till UTF-8",
-    "PHP charset is not set to UTF-8. This can cause major issues with non-ASCII characters in file names. We highly recommend to change the value of 'default_charset' php.ini to 'UTF-8'." : "PHP-teckenuppsättning är inte satt till UTF-8. Detta kan orsaka stora problem med icke-ASCII-tecken i filnamn. Vi rekommenderar starkt att ändra värdet \"default_charset\" i php.ini till \"UTF-8\".",
-    "Locale not working" : "\"Locale\" fungerar inte",
     "System locale can not be set to a one which supports UTF-8." : "Systemspråk kan inte ställas in till ett som stödjer UTF-8.",
     "This means that there might be problems with certain characters in file names." : "Detta betyder att där kan komma att uppstå problem med vissa tecken i filnamn.",
     "We strongly suggest installing the required packages on your system to support one of the following locales: %s." : "Vi rekommenderar starkt att installera de nödvändiga paketen på ditt system för att stödja en av följande språkversioner: %s.",
-    "URL generation in notification emails" : "URL-generering i notifieringsmejl",
     "If your installation is not installed in the root of the domain and uses system cron, there can be issues with the URL generation. To avoid these problems, please set the \"overwrite.cli.url\" option in your config.php file to the webroot path of your installation (Suggested: \"%s\")" : "Om din installation inte installerades på roten av domänen och använder system cron så kan det uppstå problem med URL-genereringen. För att undvika dessa problem, var vänlig sätt \"overwrite.cli.url\"-inställningen i din config.php-fil till webbrotsökvägen av din installation (Föreslagen: \"%s\")",
-    "Configuration Checks" : "Konfigurationskontroller",
-    "No problems found" : "Inga problem hittades",
-    "Please double check the <a href='%s'>installation guides</a>." : "Var god kontrollera <a href='%s'>installationsguiden</a>.",
-    "Last cron was executed at %s." : "Sista cron kördes vid %s",
-    "Last cron was executed at %s. This is more than an hour ago, something seems wrong." : "Sista cron kördes vid %s. Detta är mer än en timme sedan, något verkar fel.",
-    "Cron was not executed yet!" : "Cron kördes inte ännu!",
-    "Execute one task with each page loaded" : "Exekvera en uppgift vid varje sidladdning",
-    "cron.php is registered at a webcron service to call cron.php every 15 minutes over http." : "cron.php är registrerad som en webcron service att ropa på cron.php varje 15 minuter över http.",
-    "Use system's cron service to call the cron.php file every 15 minutes." : "Använd systemets cron-tjänst för att anropa cron.php var 15:e minut.",
     "Allow apps to use the Share API" : "Tillåt applikationer att använda delat API",
     "Allow users to share via link" : "Tillåt användare att dela via länk",
     "Enforce password protection" : "Tillämpa lösenordskydd",
@@ -149,13 +113,13 @@ OC.L10N.register(
     "Allow users to send mail notification for shared files to other users" : "Tillåt användare att skicka mejlnotifiering för delade filer till andra användare",
     "Exclude groups from sharing" : "Exkludera grupp från att dela",
     "These groups will still be able to receive shares, but not to initiate them." : "Dessa grupper kommer fortfarande kunna ta emot delningar, men inte skapa delningar.",
-    "Enforce HTTPS" : "Kräv HTTPS",
-    "Forces the clients to connect to %s via an encrypted connection." : "Tvingar klienterna att ansluta till %s via en krypterad anslutning.",
-    "Enforce HTTPS for subdomains" : "Framtvinga HTTPS för underdomäner",
-    "Forces the clients to connect to %s and subdomains via an encrypted connection." : "Tvingar klienter att ansluta till %s och underdomäner via en krypterad anslutning.",
-    "Please connect to your %s via HTTPS to enable or disable the SSL enforcement." : "Anslut till din %s via HTTPS för att aktivera/deaktivera SSL",
+    "Cron was not executed yet!" : "Cron kördes inte ännu!",
+    "Execute one task with each page loaded" : "Exekvera en uppgift vid varje sidladdning",
+    "cron.php is registered at a webcron service to call cron.php every 15 minutes over http." : "cron.php är registrerad som en webcron service att ropa på cron.php varje 15 minuter över http.",
+    "Use system's cron service to call the cron.php file every 15 minutes." : "Använd systemets cron-tjänst för att anropa cron.php var 15:e minut.",
     "This is used for sending out notifications." : "Detta används för att skicka ut notifieringar.",
     "Send mode" : "Sändningsläge",
+    "Encryption" : "Kryptering",
     "From address" : "Från adress",
     "mail" : "mail",
     "Authentication method" : "Autentiseringsmetod",
@@ -172,42 +136,28 @@ OC.L10N.register(
     "Download logfile" : "Ladda ner loggfil",
     "More" : "Mer",
     "Less" : "Mindre",
-    "The logfile is bigger than 100MB. Downloading it may take some time!" : "Loggfilen är större än 100MB. Att ladda ner den kan ta lite tid!",
     "Version" : "Version",
-    "Developed by the <a href=\"http://ownCloud.org/contact\" target=\"_blank\">ownCloud community</a>, the <a href=\"https://github.com/owncloud\" target=\"_blank\">source code</a> is licensed under the <a href=\"http://www.gnu.org/licenses/agpl-3.0.html\" target=\"_blank\"><abbr title=\"Affero General Public License\">AGPL</abbr></a>." : "Utvecklad av <a href=\"http://ownCloud.org/contact\" target=\"_blank\">ownCloud Community</a>, <a href=\"https://github.com/owncloud\" target=\"_blank\">källkoden</a> är licenserad under <a href=\"http://www.gnu.org/licenses/agpl-3.0.html\" target=\"_blank\"><abbr title=\"Affero General Public License\">AGPL</abbr></a>.",
-    "More apps" : "Fler appar",
-    "Add your app" : "Lägg till din app",
     "by" : "av",
     "licensed" : "licensierad",
     "Documentation:" : "Dokumentation:",
-    "User Documentation" : "Användardokumentation",
-    "Admin Documentation" : "Administratörsdokumentation",
     "This app cannot be installed because the following dependencies are not fulfilled:" : "Denna applikation kan inte installeras då följande beroenden inte är uppfyllda: %s",
-    "Update to %s" : "Uppdatera till %s",
     "Enable only for specific groups" : "Aktivera endast för specifika grupper",
     "Uninstall App" : "Avinstallera applikation",
     "Hey there,<br><br>just letting you know that you now have an %s account.<br><br>Your username: %s<br>Access it: <a href=\"%s\">%s</a><br><br>" : "Hej där,<br><br>vill bara informera dig om att du nu har ett %s konto.<br><br>Ditt användarnamn: %s<br>Accessa det genom: <a href=\"%s\">%s</a><br><br>",
     "Cheers!" : "Ha de fint!",
     "Hey there,\n\njust letting you know that you now have an %s account.\n\nYour username: %s\nAccess it: %s\n\n" : "Hej där,\n\nvill bara informera dig om att du nu har ett %s konto.\n\nDitt användarnamn: %s\nAccessa det genom: %s\n",
-    "Administrator Documentation" : "Administratörsdokumentation",
-    "Online Documentation" : "Onlinedokumentation",
     "Forum" : "Forum",
-    "Bugtracker" : "Bugtracker",
-    "Commercial Support" : "Kommersiell support",
     "Get the apps to sync your files" : "Skaffa apparna för att synkronisera dina filer",
     "Desktop client" : "Skrivbordsklient",
     "Android app" : "Android-app",
     "iOS app" : "iOS-app",
-    "If you want to support the project\n\t\t<a href=\"https://owncloud.org/contribute\"\n\t\t\ttarget=\"_blank\">join development</a>\n\t\tor\n\t\t<a href=\"https://owncloud.org/promote\"\n\t\t\ttarget=\"_blank\">spread the word</a>!" : "Om du vill stödja projektet\n<a href=\"https://owncloud.org/contribute\"\n\t\t\ttarget=\"_blank\">hjälp till med utvecklingen</a>\n\t\teller\n\t\t<a href=\"https://owncloud.org/promote\"\n\t\t\ttarget=\"_blank\">sprid budskapet vidare</a>!",
     "Show First Run Wizard again" : "Visa Första uppstarts-guiden igen",
     "You have used <strong>%s</strong> of the available <strong>%s</strong>" : "Du har använt <strong>%s</strong> av tillgängliga <strong>%s</strong>",
     "Password" : "Lösenord",
-    "Your password was changed" : "Ditt lösenord har ändrats",
     "Unable to change your password" : "Kunde inte ändra ditt lösenord",
     "Current password" : "Nuvarande lösenord",
     "New password" : "Nytt lösenord",
     "Change password" : "Ändra lösenord",
-    "Full Name" : "Hela namnet",
     "No display name set" : "Inget visningsnamn angivet",
     "Email" : "E-post",
     "Your email address" : "Din e-postadress",
@@ -217,7 +167,6 @@ OC.L10N.register(
     "Upload new" : "Ladda upp ny",
     "Select new from Files" : "Välj ny från filer",
     "Remove image" : "Radera bild",
-    "Either png or jpg. Ideally square but you will be able to crop it." : "Antingen png eller jpg. Helst fyrkantig, men du kommer att kunna beskära den.",
     "Your avatar is provided by your original account." : "Din avatar tillhandahålls av ditt ursprungliga konto.",
     "Cancel" : "Avbryt",
     "Choose as profile image" : "Välj som profilbild",
@@ -227,13 +176,6 @@ OC.L10N.register(
     "Valid until" : "Giltigt till",
     "Issued By" : "Utfärdat av",
     "Valid until %s" : "Giltigt till %s",
-    "Import Root Certificate" : "Importera rotcertifikat",
-    "The encryption app is no longer enabled, please decrypt all your files" : "Krypteringsapplikationen är inte längre aktiverad, vänligen dekryptera alla dina filer",
-    "Log-in password" : "Inloggningslösenord",
-    "Decrypt all Files" : "Dekryptera alla filer",
-    "Your encryption keys are moved to a backup location. If something went wrong you can restore the keys. Only delete them permanently if you are sure that all files are decrypted correctly." : "Dina krypteringsnycklar flyttas till en backup. Om något gick fel kan du återställa nycklarna. Bara ta bort dem permanent om du är säker på att alla filer dekrypteras korrekt.",
-    "Restore Encryption Keys" : "Återställ krypteringsnycklar",
-    "Delete Encryption Keys" : "Radera krypteringsnycklar",
     "Show storage location" : "Visa lagringsplats",
     "Show last log in" : "Visa senaste inloggning",
     "Show user backend" : "Visa användar-back-end",
@@ -244,7 +186,6 @@ OC.L10N.register(
     "Create" : "Skapa",
     "Admin Recovery Password" : "Admin-återställningslösenord",
     "Enter the recovery password in order to recover the users files during password change" : "Ange återställningslösenordet för att återställa användarnas filer vid lösenordsbyte",
-    "Search Users" : "Sök användare",
     "Add Group" : "Lägg till Grupp",
     "Group" : "Grupp",
     "Everyone" : "Alla",
@@ -253,6 +194,7 @@ OC.L10N.register(
     "Please enter storage quota (ex: \"512 MB\" or \"12 GB\")" : "Var god skriv in lagringskvot (ex: \"512MB\" eller \"12 GB\")",
     "Unlimited" : "Obegränsad",
     "Other" : "Annat",
+    "Full Name" : "Hela namnet",
     "Group Admin for" : "Gruppadministratör för",
     "Quota" : "Kvot",
     "Storage Location" : "Lagringsplats",
